@@ -112,14 +112,11 @@ void (*composite_alphamask_alpha_to_packed4444_scanline)( uint8_t *output,
 void (*premultiply_packed4444_scanline)( uint8_t *output, uint8_t *input, int width );
 void (*blend_packed422_scanline)( uint8_t *output, uint8_t *src1,
                                   uint8_t *src2, int width, int pos );
-void (*filter_luma_121_packed422_inplace_scanline)( uint8_t *data, int width );
-void (*filter_luma_14641_packed422_inplace_scanline)( uint8_t *data, int width );
 unsigned int (*diff_factor_packed422_scanline)( uint8_t *cur, uint8_t *old, int width );
 unsigned int (*comb_factor_packed422_scanline)( uint8_t *top, uint8_t *mid,
                                                 uint8_t *bot, int width );
 void (*kill_chroma_packed422_inplace_scanline)( uint8_t *data, int width );
 void (*mirror_packed422_inplace_scanline)( uint8_t *data, int width );
-void (*halfmirror_packed422_inplace_scanline)( uint8_t *data, int width );
 void (*speedy_memcpy)( void *output, const void *input, size_t size );
 void (*diff_packed422_block8x8)( pulldown_metrics_t *m, uint8_t *old,
                                  uint8_t *new, int os, int ns );
@@ -154,10 +151,6 @@ void (*rgb24_to_packed444_rec601_scanline)( uint8_t *output,
 void (*rgba32_to_packed4444_rec601_scanline)( uint8_t *output,
                                               uint8_t *input,
                                               int width );
-void (*chroma_422_to_444_mpeg2_plane)( uint8_t *dst, uint8_t *src,
-                                       int width, int height );
-void (*chroma_420_to_422_mpeg2_plane)( uint8_t *dst, uint8_t *src,
-                                       int width, int height, int progressive );
 void (*invert_colour_packed422_inplace_scanline)( uint8_t *data, int width );
 void (*vfilter_chroma_121_packed422_scanline)( uint8_t *output, int width,
                                                uint8_t *m, uint8_t *t, uint8_t *b );
@@ -167,9 +160,6 @@ void (*convert_uyvy_to_yuyv_scanline)( uint8_t *uyvy_buf, uint8_t *yuyv_buf, int
 void (*composite_colour4444_alpha_to_packed422_scanline)( uint8_t *output, uint8_t *input,
                                                           int af, int y, int cb, int cr,
                                                           int width, int alpha );
-void (*pointsample_packed422_image)( uint8_t *dst, int dwidth, int dheight, int dstride,
-                                     uint8_t *src, int swidth, int sheight, int sstride );
-
 
 /**
  * result = (1 - alpha)B + alpha*F
@@ -809,30 +799,6 @@ static void invert_colour_packed422_inplace_scanline_c( uint8_t *data, int width
     }
 }
 
-/*
-// this duplicates alternate lines in alternate frames to highlight or mute
-// the effects of chroma crawl. it is not a solution or proper filter. it's
-// only for testing.
-void testing_packed422_inplace_scanline_c( uint8_t *data, int width, int scanline )
-{
-    volatile static int topbottom = 0;
-    static uint8_t scanbuffer[2048];
-
-    if( scanline <= 1 ) {
-        topbottom = scanline;
-        memcpy(scanbuffer, data, width*2);
-    }
-    if ( scanline < 10 ) {
-        printf("scanline: %d %d\n", scanline, topbottom);
-    }
-    if ( ((scanline-topbottom)/2)%2 && scanline > 1 ) {
-        memcpy(data, scanbuffer, width*2);
-    } else {
-        memcpy(scanbuffer, data, width*2);
-    }
-}
-*/
-
 static void mirror_packed422_inplace_scanline_c( uint8_t *data, int width )
 {
     int x, tmp1, tmp2;
@@ -848,52 +814,6 @@ static void mirror_packed422_inplace_scanline_c( uint8_t *data, int width )
     }
 }
 
-static void halfmirror_packed422_inplace_scanline_c( uint8_t *data, int width )
-{
-    int x;
-
-    for( x = 0; x < width; x += 2 ) {
-        data[ width + x     ] = data[ width - x     ];
-        data[ width + x + 1 ] = data[ width - x + 1 ];
-    }
-}
-
-static void filter_luma_121_packed422_inplace_scanline_c( uint8_t *data, int width )
-{
-    int r1 = 0;
-    int r2 = 0;
-
-    data += 2;
-    width -= 1;
-    while( width-- ) {
-        int s1, s2;
-        s1 = *data + r1; r1 = *data;
-        s2 = s1    + r2; r2 = s1;
-        *(data - 2) = s2 >> 2;
-        data += 2;
-    }
-}
-
-static void filter_luma_14641_packed422_inplace_scanline_c( uint8_t *data, int width )
-{
-    int r1 = 0;
-    int r2 = 0;
-    int r3 = 0;
-    int r4 = 0;
-
-    width -= 4;
-    data += 4;
-    while( width-- ) {
-        int s1, s2, s3, s4;
-        s1 = *data + r1; r1 = *data;
-        s2 = s1    + r2; r2 = s1;
-        s3 = s2    + r3; r3 = s2;
-        s4 = s3    + r4; r4 = s3;
-        *(data - 4) = s4 >> 4;
-        data += 2;
-    }
-}
-
 static void interpolate_packed422_scanline_c( uint8_t *output, uint8_t *top,
                                               uint8_t *bot, int width )
 {
@@ -907,6 +827,7 @@ static void interpolate_packed422_scanline_c( uint8_t *output, uint8_t *top,
 #ifdef ARCH_X86
 static void convert_uyvy_to_yuyv_scanline_mmx( uint8_t *uyvy_buf, uint8_t *yuyv_buf, int width )
 {
+#ifdef ARCH_386
     __asm__ __volatile__(
         "   movl      %0, %%esi         \n"
         "   movl      %1, %%edi         \n"
@@ -934,7 +855,37 @@ static void convert_uyvy_to_yuyv_scanline_mmx( uint8_t *uyvy_buf, uint8_t *yuyv_
         /* output */            :
         /* input */             : "g" (uyvy_buf), "g" (yuyv_buf), "g" (width)
         /* clobber registers */ : "cc", "edx", "esi", "edi" );
+#endif
+#ifdef ARCH_X86_64
+    __asm__ __volatile__(
+        "   movq      %0, %%rsi         \n"
+        "   movq      %1, %%rdi         \n"
+        "   xorq      %%rdx, %%rdx      \n"
+        "   movl      %2, %%edx         \n"
+        "   shrq      $3, %%rdx         \n"
 
+        /* Process 8 pixels at once */
+        "1: movq      (%%rsi), %%mm0    \n" /* mm0 = Y3V2Y2U2Y1V0Y0U0 */
+        "   movq      8(%%rsi), %%mm2   \n" /* mm2 = Y7V6Y6U6Y5V4Y4U4 */
+        "   movq      %%mm0, %%mm1      \n" /* mm1 = Y3V2Y2U2Y1V0Y0U0 */
+        "   movq      %%mm2, %%mm3      \n" /* mm3 = Y7V6Y6U6Y5V4Y4U4 */
+        "   psllw     $8, %%mm0         \n" /* mm0 = V2__U2__V0__U0__ */
+        "   psrlw     $8, %%mm1         \n" /* mm1 = __Y3__Y2__Y1__Y0 */
+        "   psllw     $8, %%mm2         \n" /* mm2 = V6__U6__V4__U4__ */
+        "   psrlw     $8, %%mm3         \n" /* mm3 = __Y7__Y6__Y5__Y4 */
+        "   por       %%mm1, %%mm0      \n" /* mm0 = V2Y3U2Y2V0Y1U0Y0 */
+        "   por       %%mm3, %%mm2      \n" /* mm2 = V6Y7U6Y6V4Y5U4Y4 */
+        "   movq      %%mm0, (%%rdi)    \n"
+        "   movq      %%mm2, 8(%%rdi)   \n"
+        "   addq      $16, %%rsi        \n"
+        "   addq      $16, %%rdi        \n"
+        "   decq      %%rdx             \n"
+        "   jnz       1b                \n"
+        "   emms                        \n"
+        /* output */            :
+        /* input */             : "g" (uyvy_buf), "g" (yuyv_buf), "g" (width)
+        /* clobber registers */ : "cc", "rdx", "rsi", "rdi" );
+#endif
     if( width & 7 ) {
         uint32_t *uyvy = (uint32_t *) uyvy_buf;
         uint32_t *yuyv = (uint32_t *) yuyv_buf;
@@ -2525,158 +2476,6 @@ static void aspect_adjust_packed4444_scanline_c( uint8_t *output,
     }
 }
 
-/* horizontal 1:2 interpolation filter */
-static void chroma_422_to_444_mpeg2_plane_c( uint8_t *dst, uint8_t *src, int width, int height )
-{
-    int i, i2, w, j, im2, im1, ip1, ip2, ip3;
-
-    w = width / 2;
-
-    for( j = 0; j < height; j++ ) {
-        for( i = 0; i < w; i++ ) {
-            i2 = i << 1;
-            im2 = (i<2) ? 0 : i-2;
-            im1 = (i<1) ? 0 : i-1;
-            ip1 = (i<w-1) ? i+1 : w-1;
-            ip2 = (i<w-2) ? i+2 : w-1;
-            ip3 = (i<w-3) ? i+3 : w-1;
-
-            /* FIR filter coefficients (*256): 21 0 -52 0 159 256 159 0 -52 0 21 */
-            /* even samples (0 0 256 0 0) */
-            dst[ i2 ] = src[i];
-
-            /* odd samples (21 -52 159 159 -52 21) */
-            dst[ i2 + 1 ] = clip255( (   21*(src[im2]+src[ip3])
-                                      -  52*(src[im1]+src[ip2]) 
-                                      + 159*(src[i]+src[ip1]) + 128 ) >> 8 );
-        }
-        src += w;
-        dst += width;
-    }
-}
-
-/* vertical 1:2 interpolation filter */
-static void chroma_420_to_422_mpeg2_plane_c( uint8_t *dst, uint8_t *src,
-                                             int width, int height, int progressive )
-{
-    int w, h, i, j, j2;
-    int jm6, jm5, jm4, jm3, jm2, jm1, jp1, jp2, jp3, jp4, jp5, jp6, jp7;
-
-    w = width / 2;
-    h = height / 2;
-
-    if( progressive ) {
-        /* intra frame */
-        for( i = 0; i < w; i++ ) {
-            for( j = 0; j < h; j++ ) {
-                j2 = j << 1;
-                jm3 = (j<3) ? 0 : j-3;
-                jm2 = (j<2) ? 0 : j-2;
-                jm1 = (j<1) ? 0 : j-1;
-                jp1 = (j<h-1) ? j+1 : h-1;
-                jp2 = (j<h-2) ? j+2 : h-1;
-                jp3 = (j<h-3) ? j+3 : h-1;
-
-                /* FIR filter coefficients (*256): 5 -21 70 228 -37 11 */
-                /* New FIR filter coefficients (*256): 3 -16 67 227 -32 7 */
-                dst[w*j2] = clip255( (    3*src[w*jm3]
-                                       - 16*src[w*jm2]
-                                       + 67*src[w*jm1]
-                                      + 227*src[w*j]
-                                       - 32*src[w*jp1]
-                                        + 7*src[w*jp2] + 128 ) >> 8 );
-
-                dst[w*(j2+1)] = clip255( (   3*src[w*jp3]
-                                          - 16*src[w*jp2]
-                                          + 67*src[w*jp1]
-                                         + 227*src[w*j]
-                                          - 32*src[w*jm1]
-                                          + 7*src[w*jm2] + 128 ) >> 8 );
-            }
-            src++;
-            dst++;
-        }
-    } else {
-        /* intra field */
-        for( i = 0; i < w; i++ ) {
-            for( j = 0; j < h; j += 2 ) {
-                j2 = j << 1;
-
-                /* top field */
-                jm6 = (j<6) ? 0 : j-6;
-                jm4 = (j<4) ? 0 : j-4;
-                jm2 = (j<2) ? 0 : j-2;
-                jp2 = (j<h-2) ? j+2 : h-2;
-                jp4 = (j<h-4) ? j+4 : h-2;
-                jp6 = (j<h-6) ? j+6 : h-2;
-
-                /* Polyphase FIR filter coefficients (*256): 2 -10 35 242 -18 5 */
-                /* New polyphase FIR filter coefficients (*256): 1 -7 30 248 -21 5 */
-                dst[w*j2] = clip255( (   1*src[w*jm6]
-                                       - 7*src[w*jm4]
-                                      + 30*src[w*jm2]
-                                     + 248*src[w*j]
-                                      - 21*src[w*jp2]
-                                       + 5*src[w*jp4] + 128 ) >> 8 );
-
-                /* Polyphase FIR filter coefficients (*256): 11 -38 192 113 -30 8 */
-                /* New polyphase FIR filter coefficients (*256):7 -35 194 110 -24 4 */
-                dst[w*(j2+2)] = clip255( ( 7*src[w*jm4]
-                                         -35*src[w*jm2]
-                                        +194*src[w*j]
-                                        +110*src[w*jp2]
-                                         -24*src[w*jp4]
-                                          +4*src[w*jp6] + 128 ) >> 8 );
-
-                /* bottom field */
-                jm5 = (j<5) ? 1 : j-5;
-                jm3 = (j<3) ? 1 : j-3;
-                jm1 = (j<1) ? 1 : j-1;
-                jp1 = (j<h-1) ? j+1 : h-1;
-                jp3 = (j<h-3) ? j+3 : h-1;
-                jp5 = (j<h-5) ? j+5 : h-1;
-                jp7 = (j<h-7) ? j+7 : h-1;
-
-                /* Polyphase FIR filter coefficients (*256): 11 -38 192 113 -30 8 */
-                /* New polyphase FIR filter coefficients (*256):7 -35 194 110 -24 4 */
-                dst[w*(j2+1)] = clip255( ( 7*src[w*jp5]
-                                         -35*src[w*jp3]
-                                        +194*src[w*jp1]
-                                        +110*src[w*jm1]
-                                         -24*src[w*jm3]
-                                          +4*src[w*jm5] + 128 ) >> 8 );
-
-                dst[w*(j2+3)] = clip255( (  1*src[w*jp7]
-                                           -7*src[w*jp5]
-                                          +30*src[w*jp3]
-                                         +248*src[w*jp1]
-                                          -21*src[w*jm1]
-                                           +5*src[w*jm3] + 128 ) >> 8 );
-            }
-            src++;
-            dst++;
-        }
-    }
-}
-
-void pointsample_packed422_image_c( uint8_t *dst, int dwidth, int dheight, int dstride,
-                                    uint8_t *src, int swidth, int sheight, int sstride )
-{
-    int x, y;
-    int qdwidth = (dwidth * 2) / 4;
-    int qswidth = (swidth * 2) / 4;
-
-    for( y = 0; y < dheight; y++ ) {
-        uint8_t *curdst = dst + (y * dstride);
-        uint8_t *cursrc = src + (((y * (sheight - 1)) / (dheight - 1)) * sstride);
-
-        for( x = 0; x < qdwidth; x++ ) {
-            uint8_t *srcpx = cursrc + (((x * (qswidth - 1)) / (qdwidth - 1)) * 4);
-            *((uint32_t *) (curdst + (x * 4))) = *((uint32_t *) srcpx);
-        }
-    }
-}
-
 static uint32_t speedy_accel;
 
 void setup_speedy_calls( uint32_t accel, int verbose )
@@ -2693,13 +2492,10 @@ void setup_speedy_calls( uint32_t accel, int verbose )
     composite_alphamask_alpha_to_packed4444_scanline = composite_alphamask_alpha_to_packed4444_scanline_c;
     premultiply_packed4444_scanline = premultiply_packed4444_scanline_c;
     blend_packed422_scanline = blend_packed422_scanline_c;
-    filter_luma_121_packed422_inplace_scanline = filter_luma_121_packed422_inplace_scanline_c;
-    filter_luma_14641_packed422_inplace_scanline = filter_luma_14641_packed422_inplace_scanline_c;
     comb_factor_packed422_scanline = 0;
     diff_factor_packed422_scanline = diff_factor_packed422_scanline_c;
     kill_chroma_packed422_inplace_scanline = kill_chroma_packed422_inplace_scanline_c;
     mirror_packed422_inplace_scanline = mirror_packed422_inplace_scanline_c;
-    halfmirror_packed422_inplace_scanline = halfmirror_packed422_inplace_scanline_c;
     speedy_memcpy = speedy_memcpy_c;
     diff_packed422_block8x8 = diff_packed422_block8x8_c;
     a8_subpix_blit_scanline = a8_subpix_blit_scanline_c;
@@ -2713,14 +2509,11 @@ void setup_speedy_calls( uint32_t accel, int verbose )
     packed444_to_rgb24_rec601_scanline = packed444_to_rgb24_rec601_scanline_c;
     rgb24_to_packed444_rec601_scanline = rgb24_to_packed444_rec601_scanline_c;
     rgba32_to_packed4444_rec601_scanline = rgba32_to_packed4444_rec601_scanline_c;
-    chroma_422_to_444_mpeg2_plane = chroma_422_to_444_mpeg2_plane_c;
-    chroma_420_to_422_mpeg2_plane = chroma_420_to_422_mpeg2_plane_c;
     invert_colour_packed422_inplace_scanline = invert_colour_packed422_inplace_scanline_c;
     vfilter_chroma_121_packed422_scanline = vfilter_chroma_121_packed422_scanline_c;
     vfilter_chroma_332_packed422_scanline = vfilter_chroma_332_packed422_scanline_c;
     convert_uyvy_to_yuyv_scanline = convert_uyvy_to_yuyv_scanline_c;
     composite_colour4444_alpha_to_packed422_scanline = composite_colour4444_alpha_to_packed422_scanline_c;
-    pointsample_packed422_image = pointsample_packed422_image_c;
 
 #ifdef ARCH_X86
     if( speedy_accel & MM_ACCEL_X86_MMXEXT ) {
