@@ -39,12 +39,13 @@ struct station_info_s
     char network_name[ 33 ];
     char network_call_letters[ 7 ];
     char norm[ 10 ];
+    char audio[ 10 ];
     char xmltvid[ 128 ];
     const band_t *band;
     const band_entry_t *channel;
     int brightness;
     int contrast;
-    int colour;
+    int saturation;
     int hue;
     int finetune;
 
@@ -65,6 +66,7 @@ struct station_mgr_s
     char band_and_frequency[ 1024 ];
     char stationrc[ 255 ];
     char *norm;
+    char audio[ 10 ];
     char *table;
 };
 
@@ -115,10 +117,11 @@ static station_info_t *station_info_new( int pos, const char *name,
         memset( i->network_name, 0, sizeof( i->network_name ) );
         memset( i->network_call_letters, 0, sizeof( i->network_call_letters ) );
         memset( i->norm, 0, sizeof( i->norm ) );
+        memset( i->audio, 0, sizeof( i->audio ) );
         memset( i->xmltvid, 0, sizeof( i->xmltvid ) );
         i->brightness = -1;
         i->contrast = -1;
-        i->colour = -1;
+        i->saturation = -1;
         i->hue = -1;
         i->finetune = 0;
 
@@ -222,6 +225,7 @@ int station_readconfig( station_mgr_t *mgr )
     xmlDocPtr doc;
     xmlNodePtr station;
     xmlNodePtr list;
+    xmlChar *list_audio;
 
     if( !file_is_openable_for_read( mgr->stationrc ) ) {
         /* There is no file to try to read, so forget it. */
@@ -258,6 +262,11 @@ int station_readconfig( station_mgr_t *mgr )
         xmlFreeDoc( doc );
         return 0;
     }
+    list_audio = xmlGetProp( list, BAD_CAST "audio" );
+    if( list_audio ) {
+        snprintf( mgr->audio, sizeof( mgr->audio ), "%s", list_audio );
+        xmlFree( list_audio );
+    }
 
     station = list->xmlChildrenNode;
     while( station ) {
@@ -270,13 +279,18 @@ int station_readconfig( station_mgr_t *mgr )
             xmlChar *network = xmlGetProp( station, BAD_CAST "network" );
             xmlChar *call = xmlGetProp( station, BAD_CAST "call" );
             xmlChar *norm = xmlGetProp( station, BAD_CAST "norm" );
+            xmlChar *audio = xmlGetProp( station, BAD_CAST "audio" );
             xmlChar *brightness = xmlGetProp( station, BAD_CAST "brightness" );
             xmlChar *contrast = xmlGetProp( station, BAD_CAST "contrast" );
-            xmlChar *colour = xmlGetProp( station, BAD_CAST "colour" );
+            xmlChar *saturation = xmlGetProp( station, BAD_CAST "colour" );
             xmlChar *hue = xmlGetProp( station, BAD_CAST "hue" );
             xmlChar *finetune = xmlGetProp( station, BAD_CAST "finetune" );
             xmlChar *xmltvid = xmlGetProp( station, BAD_CAST "xmltvid" );
 
+            if( !saturation ) {
+                saturation = xmlGetProp( station, BAD_CAST "saturation" );
+            }
+
             /* Only band and channel are required. */
             if( band && channel ) {
                 int pos, active;
@@ -299,9 +313,20 @@ int station_readconfig( station_mgr_t *mgr )
                 } else {
                     station_set_current_norm( mgr, mgr->norm );
                 }
+                if( audio ) {
+                    if( !strcasecmp( (char *) audio, "auto" ) ) {
+                        station_set_current_audio_norm( mgr, -1 );
+                    } else if( !strcasecmp( (char *) audio, "dk" ) ) {
+                        station_set_current_audio_norm( mgr, 1 );
+                    } else {
+                        station_set_current_audio_norm( mgr, 0 );
+                    }
+                } else {
+                    station_set_current_audio_norm( mgr, -1 );
+                }
                 if( brightness ) station_set_current_brightness( mgr, atoi( (char *) brightness ) );
                 if( contrast ) station_set_current_contrast( mgr, atoi( (char *) contrast ) );
-                if( colour ) station_set_current_colour( mgr, atoi( (char *) colour ) );
+                if( saturation ) station_set_current_saturation( mgr, atoi( (char *) saturation ) );
                 if( hue ) station_set_current_hue( mgr, atoi( (char *) hue ) );
                 if( finetune ) station_set_current_finetune( mgr, atoi( (char *) finetune ) );
                 if( xmltvid ) station_set_current_xmltv_id( mgr, (char *) xmltvid );
@@ -311,9 +336,10 @@ int station_readconfig( station_mgr_t *mgr )
             if( finetune ) xmlFree( finetune );
             if( brightness ) xmlFree( brightness );
             if( contrast ) xmlFree( contrast );
-            if( colour ) xmlFree( colour );
+            if( saturation ) xmlFree( saturation );
             if( hue ) xmlFree( hue );
             if( norm ) xmlFree( norm );
+            if( audio ) xmlFree( audio );
             if( network ) xmlFree( network );
             if( call ) xmlFree( call );
             if( name ) xmlFree( name );
@@ -351,6 +377,7 @@ station_mgr_t *station_new( const char *norm, const char *table,
         free( mgr );
         return 0;
     }
+    sprintf( mgr->audio, "bg" );
 
     mgr->table = strdup( table );
     if( !mgr->table ) {
@@ -473,6 +500,27 @@ int station_set( station_mgr_t *mgr, int pos )
     return 0;
 }
 
+int station_set_by_name( station_mgr_t *mgr, const char *name )
+{
+    station_info_t *rp = mgr->first;
+
+    if( rp ) {
+        do {
+            if( !strcasecmp( rp->name, name ) ) {
+                if( mgr->current == rp ) {
+                    return 0;
+                }
+                mgr->last_channel = station_get_current_id( mgr );
+                mgr->current = rp;
+                return 1;
+            }
+            rp = rp->next;
+        } while( rp != mgr->first );
+    }
+
+    return 0;
+}
+
 void station_inc( station_mgr_t *mgr )
 {
     mgr->last_channel = station_get_current_id( mgr );
@@ -570,6 +618,20 @@ void station_toggle_us_cable_mode( station_mgr_t *mgr )
     mgr->us_cable_mode = (mgr->us_cable_mode + 1) % 3;
 }
 
+void station_set_default_audio_norm( station_mgr_t *mgr, int dk )
+{
+    if( dk ) {
+        sprintf( mgr->audio, "dk" );
+    } else {
+        sprintf( mgr->audio, "bg" );
+    }
+}
+
+int station_get_default_audio_norm( station_mgr_t *mgr )
+{
+    return !strcasecmp( mgr->audio, "dk" );
+}
+
 int station_get_current_frequency( station_mgr_t *mgr )
 {
     if( !mgr->current ) {
@@ -726,6 +788,36 @@ void station_set_current_norm( station_mgr_t *mgr, const char *norm )
     }
 }
 
+int station_get_current_audio_norm( station_mgr_t *mgr )
+{
+    const char *str;
+
+    if( mgr->current ) {
+        if( !strcasecmp( mgr->current->audio, "auto" ) ) {
+            str = mgr->audio;
+        } else {
+            str = mgr->current->audio;
+        }
+    } else {
+        str = mgr->audio;
+    }
+
+    return !strcasecmp( str, "dk" );
+}
+
+void station_set_current_audio_norm( station_mgr_t *mgr, int dk )
+{
+    if( mgr->current ) {
+        if( dk < 0 ) {
+            sprintf( mgr->current->audio, "auto" );
+        } else if( dk ) {
+            sprintf( mgr->current->audio, "dk" );
+        } else {
+            sprintf( mgr->current->audio, "bg" );
+        }
+    }
+}
+
 void station_set_current_brightness( station_mgr_t *mgr, int val )
 {
     if( mgr->current ) {
@@ -740,10 +832,10 @@ void station_set_current_contrast( station_mgr_t *mgr, int val )
     }
 }
 
-void station_set_current_colour( station_mgr_t *mgr, int val )
+void station_set_current_saturation( station_mgr_t *mgr, int val )
 {
     if( mgr->current ) {
-        mgr->current->colour = val;
+        mgr->current->saturation = val;
     }
 }
 
@@ -772,10 +864,10 @@ int station_get_current_contrast( station_mgr_t *mgr )
     }
 }
 
-int station_get_current_colour( station_mgr_t *mgr )
+int station_get_current_saturation( station_mgr_t *mgr )
 {
     if( mgr->current ) {
-        return mgr->current->colour;
+        return mgr->current->saturation;
     } else {
         return -1;
     }
@@ -982,6 +1074,7 @@ int station_writeconfig( station_mgr_t *mgr )
         xmlNewProp( list, BAD_CAST "norm", BAD_CAST mgr->norm );
         xmlNewProp( list, BAD_CAST "frequencies", BAD_CAST mgr->table );
     }
+    xmlSetProp( list, BAD_CAST "audio", BAD_CAST mgr->audio );
 
     do {
         xmlNodePtr node = find_station( list->xmlChildrenNode, rp->name );
@@ -1011,6 +1104,9 @@ int station_writeconfig( station_mgr_t *mgr )
         if( *(rp->norm) ) {
             xmlSetProp( node, BAD_CAST "norm", BAD_CAST rp->norm );
         }
+        if( *(rp->audio) ) {
+            xmlSetProp( node, BAD_CAST "audio", BAD_CAST rp->audio );
+        }
         if( rp->brightness >= 0 ) {
             snprintf( buf, sizeof( buf ), "%d", rp->brightness );
             xmlSetProp( node, BAD_CAST "brightness", BAD_CAST buf );
@@ -1019,9 +1115,9 @@ int station_writeconfig( station_mgr_t *mgr )
             snprintf( buf, sizeof( buf ), "%d", rp->contrast );
             xmlSetProp( node, BAD_CAST "contrast", BAD_CAST buf );
         }
-        if( rp->colour >= 0 ) {
-            snprintf( buf, sizeof( buf ), "%d", rp->colour );
-            xmlSetProp( node, BAD_CAST "colour", BAD_CAST buf );
+        if( rp->saturation >= 0 ) {
+            snprintf( buf, sizeof( buf ), "%d", rp->saturation );
+            xmlSetProp( node, BAD_CAST "saturation", BAD_CAST buf );
         }
         if( rp->hue >= 0 ) {
             snprintf( buf, sizeof( buf ), "%d", rp->hue );
