@@ -29,9 +29,11 @@
 #include <unistd.h>
 #include <dirent.h>
 #include <langinfo.h>
+#include <locale.h>
 #include <pwd.h>
 #include <errno.h>
 #include <libxml/parser.h>
+#include <math.h>
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -47,36 +49,33 @@
 #include "utils.h"
 
 #define MAX_KEYSYMS 350
-#define MAX_BUTTONS 10
-
-/* Mode list. */
-typedef struct tvtime_modelist_s tvtime_modelist_t;
+#define MAX_BUTTONS 15
 
 struct config_s
 {
-    int outputheight;
+    char *geometry;
     int verbose;
     int aspect;
     int debug;
     int fullscreen;
+    int alwaysontop;
     int priority;
     int ntsc_mode;
     int send_fields;
-    char *output_driver;
-    int apply_luma_correction;
-    double luma_correction;
-    int useposition;
     int fspos;
-    int x;
-    int y;
     int picsaverestore;
     int brightness;
     int contrast;
-    int colour;
+    int saturation;
     int hue;
     int invert;
     int cc;
     int mirror;
+    int boost;
+    int quiet_screenshots;
+    int unmute_volume;
+    int muted;
+    int mute_on_exit;
 
     int keymap[ 8 * MAX_KEYSYMS ];
     char *keymap_arg[ 8 * MAX_KEYSYMS ];
@@ -96,14 +95,12 @@ struct config_s
     char *timeformat;
     char *audiomode;
     char *xmltvfile;
+    char *xmltvlanguage;
     unsigned int channel_text_rgb;
     unsigned int other_text_rgb;
 
     uid_t uid;
 
-    char *rvr_filename;
-    char *mpeg_filename;
-
     char *mixerdev;
 
     char *deinterlace_method;
@@ -121,50 +118,8 @@ struct config_s
 
     char *config_filename;
     xmlDocPtr doc;
-
-    int nummodes;
-    tvtime_modelist_t *modelist;
 };
 
-/* Mode list. */
-struct tvtime_modelist_s
-{
-    config_t settings;
-    char *name;
-    tvtime_modelist_t *next;
-};
-
-static void copy_config( config_t *dest, config_t *src )
-{
-    (*dest) = (*src);
-
-    /* Some of these I am keeping invalid for now. */
-    dest->v4ldev = 0;
-    dest->vbidev = 0;
-    dest->rvr_filename = 0;
-    dest->mpeg_filename = 0;
-    dest->mixerdev = 0;
-    dest->config_filename = 0;
-    dest->modelist = 0;
-    dest->nummodes = 0;
-    dest->doc = 0;
-    dest->output_driver = 0;
-    dest->audiomode = 0;
-    dest->xmltvfile = 0;
-
-    /* Useful strings must be copied. */
-    dest->norm = strdup( src->norm );
-    dest->freq = strdup( src->freq );
-    dest->ssdir = strdup( src->ssdir );
-    dest->timeformat = strdup( src->timeformat );
-    dest->deinterlace_method = strdup( src->timeformat );
-
-    memset( dest->keymap_arg, 0, sizeof( dest->keymap_arg ) );
-    memset( dest->keymapmenu_arg, 0, sizeof( dest->keymapmenu_arg ) );
-    memset( dest->buttonmap_arg, 0, sizeof( dest->buttonmap_arg ) );
-    memset( dest->buttonmapmenu_arg, 0, sizeof( dest->buttonmapmenu_arg ) );
-}
-
 static unsigned int parse_colour( const char *str )
 {
     unsigned int a, r, g, b;
@@ -223,24 +178,9 @@ static void parse_option( config_t *ct, xmlNodePtr node )
     if( name && value ) {
         char *curval = (char *) value;
 
-        if( !xmlStrcasecmp( name, BAD_CAST "OutputHeight" ) ) {
-            if( tolower( curval[ 0 ] ) == 'f' ) {
-                ct->outputheight = -1;
-            } else {
-                ct->outputheight = atoi( curval );
-            }
-        }
-
-        if( !xmlStrcasecmp( name, BAD_CAST "UseWindowPosition" ) ) {
-            ct->useposition = atoi( curval );
-        }
-
-        if( !xmlStrcasecmp( name, BAD_CAST "WindowX" ) ) {
-            ct->x = atoi( curval );
-        }
-
-        if( !xmlStrcasecmp( name, BAD_CAST "WindowY" ) ) {
-            ct->y = atoi( curval );
+        if( !xmlStrcasecmp( name, BAD_CAST "WindowGeometry" ) ) {
+            if( ct->geometry ) free( ct->geometry );
+            ct->geometry = strdup( curval );
         }
 
         if( !xmlStrcasecmp( name, BAD_CAST "InputWidth" ) ) {
@@ -264,7 +204,11 @@ static void parse_option( config_t *ct, xmlNodePtr node )
         }
 
         if( !xmlStrcasecmp( name, BAD_CAST "DefaultColour" ) ) {
-            ct->colour = atoi( curval );
+            ct->saturation = atoi( curval );
+        }
+
+        if( !xmlStrcasecmp( name, BAD_CAST "DefaultSaturation" ) ) {
+            ct->saturation = atoi( curval );
         }
 
         if( !xmlStrcasecmp( name, BAD_CAST "DefaultHue" ) ) {
@@ -279,6 +223,26 @@ static void parse_option( config_t *ct, xmlNodePtr node )
             ct->mirror = atoi( curval );
         }
 
+        if( !xmlStrcasecmp( name, BAD_CAST "AudioBoost" ) ) {
+            ct->boost = atoi( curval );
+        }
+
+        if( !xmlStrcasecmp( name, BAD_CAST "QuietScreenshots" ) ) {
+            ct->quiet_screenshots = atoi( curval );
+        }
+
+        if( !xmlStrcasecmp( name, BAD_CAST "UnmuteVolume" ) ) {
+            ct->unmute_volume = atoi( curval );
+        }
+
+        if( !xmlStrcasecmp( name, BAD_CAST "Muted" ) ) {
+            ct->muted = atoi( curval );
+        }
+
+        if( !xmlStrcasecmp( name, BAD_CAST "MuteOnExit" ) ) {
+            ct->mute_on_exit = atoi( curval );
+        }
+
         if( !xmlStrcasecmp( name, BAD_CAST "ShowCC" ) ) {
             ct->cc = atoi( curval );
         }
@@ -297,9 +261,9 @@ static void parse_option( config_t *ct, xmlNodePtr node )
             ct->xmltvfile = expand_user_path( curval );
         }
 
-        if( !xmlStrcasecmp( name, BAD_CAST "OutputDriver" ) ) {
-            if( ct->output_driver ) free( ct->output_driver );
-            ct->output_driver = strdup( curval );
+        if( !xmlStrcasecmp( name, BAD_CAST "XMLTVLanguage" ) ) {
+            if( ct->xmltvlanguage ) free( ct->xmltvlanguage );
+            ct->xmltvlanguage = strdup( curval );
         }
 
         if( !xmlStrcasecmp( name, BAD_CAST "FullscreenPosition" ) ) {
@@ -320,14 +284,6 @@ static void parse_option( config_t *ct, xmlNodePtr node )
             ct->debug = atoi( curval );
         }
 
-        if( !xmlStrcasecmp( name, BAD_CAST "ApplyLumaCorrection" ) ) {
-            ct->apply_luma_correction = atoi( curval );
-        }
-
-        if( !xmlStrcasecmp( name, BAD_CAST "LumaCorrection" ) ) {
-            ct->luma_correction = atof( curval );
-        }
-
         if( !xmlStrcasecmp( name, BAD_CAST "V4LDevice" ) ) {
             if( ct->v4ldev ) free( ct->v4ldev );
             ct->v4ldev = strdup( curval );
@@ -354,6 +310,10 @@ static void parse_option( config_t *ct, xmlNodePtr node )
             ct->fullscreen = atoi( curval );
         }
 
+        if( !xmlStrcasecmp( name, BAD_CAST "AlwaysOnTop" ) ) {
+            ct->alwaysontop = atoi( curval );
+        }
+
         if( !xmlStrcasecmp( name, BAD_CAST "FramerateMode" ) ) {
             ct->framerate = atoi( curval );
         }
@@ -414,7 +374,18 @@ static void parse_option( config_t *ct, xmlNodePtr node )
         }
 
         if( !xmlStrcasecmp( name, BAD_CAST "Overscan" ) ) {
+            char prevloc[ 256 ];
+
+            /* Make sure floating point numbers are always written to
+             * the config file in "C" locale format.
+             */
+            strncpy( prevloc, setlocale( LC_NUMERIC, NULL ), sizeof prevloc );
+            setlocale( LC_NUMERIC, "C" );
             ct->overscan = ( atof( curval ) / 2.0 ) / 100.0;
+            if( !isnormal( ct->overscan ) ) {
+                ct->overscan = 0.0;
+            }
+            setlocale( LC_NUMERIC, prevloc );
         }
 
         if( !xmlStrcasecmp( name, BAD_CAST "MixerDevice" ) ) {
@@ -481,36 +452,6 @@ static void parse_bind( config_t *ct, xmlNodePtr node )
     if( arg ) xmlFree( arg );
 }
 
-static void parse_mode( config_t *ct, xmlNodePtr node )
-{
-    xmlChar *name = xmlGetProp( node, BAD_CAST "name" );
-
-    if( name ) {
-        tvtime_modelist_t *mode = malloc( sizeof( tvtime_modelist_t ) );
-        if( mode ) {
-
-            /* Start with the default settings from the config file. */
-            copy_config( &(mode->settings), ct );
-
-            node = node->xmlChildrenNode;
-            while( node ) {
-                if( !xmlIsBlankNode( node ) ) {
-                    if( !xmlStrcasecmp( node->name, BAD_CAST "option" ) ) {
-                        parse_option( &(mode->settings), node );
-                    }
-                }
-                node = node->next;
-            }
-
-            mode->name = strdup( (char *) name );
-            mode->next = ct->modelist;
-            ct->modelist = mode;
-            ct->nummodes++;
-        }
-        xmlFree( name );
-    }
-}
-
 static int conf_xml_parse( config_t *ct, char *configFile )
 {
     xmlDocPtr doc;
@@ -546,8 +487,6 @@ static int conf_xml_parse( config_t *ct, char *configFile )
                 parse_option( ct, node );
             } else if( !xmlStrcasecmp( node->name, BAD_CAST "bind" ) ) {
                 parse_bind( ct, node );
-            } else if( !xmlStrcasecmp( node->name, BAD_CAST "mode" ) ) {
-                parse_mode( ct, node );
             }
         }
         node = node->next;
@@ -644,8 +583,6 @@ static void print_usage( char **argv )
     lfputs( _("  -b, --vbidevice=DEVICE     VBI device (defaults to /dev/vbi0).\n"), stderr );
     lfputs( _("  -c, --channel=CHANNEL      Tune to the specified channel on startup.\n"), stderr );
     lfputs( _("  -d, --device=DEVICE        video4linux device (defaults to /dev/video0).\n"), stderr );
-    lfputs( _("  -D, --driver=NAME          Output driver to use: Xv, DirectFB, mga,\n"
-              "                             matroxtv or SDL (defaults to Xv).\n"), stderr );
     lfputs( _("  -f, --frequencies=NAME     The frequency table to use for the tuner.\n"
               "                             (defaults to us-cable).\n\n"
               "                             Valid values are:\n"
@@ -664,7 +601,7 @@ static void print_usage( char **argv )
               "                                 custom (first run tvtime-scanner)\n\n"), stderr );
     lfputs( _("  -F, --configfile=FILE      Additional config file to load settings from.\n"), stderr );
     lfputs( _("  -h, --help                 Show this help message.\n"), stderr );
-    lfputs( _("  -H, --height=HEIGHT        Output window height (defaults to 576 pixels).\n"), stderr );
+    lfputs( _("  -g, --geometry=GEOMETRY    Sets the output window size.\n"), stderr );
     lfputs( _("  -i, --input=INPUTNUM       video4linux input number (defaults to 0).\n"), stderr );
     lfputs( _("  -I, --inputwidth=SAMPLING  Horizontal resolution of input\n"
               "                             (defaults to 720 pixels).\n"), stderr );
@@ -674,13 +611,12 @@ static void print_usage( char **argv )
     lfputs( _("  -n, --norm=NORM            The norm to use for the input.  tvtime supports:\n"
               "                             NTSC, NTSC-JP, SECAM, PAL, PAL-Nc, PAL-M,\n"
               "                             PAL-N or PAL-60 (defaults to NTSC).\n"), stderr );
-    lfputs( _("  -p, --fspos=POS            Set the fullscreen position: top, bottom or\n"
-              "                             centre (default).\n"), stderr );
-    lfputs( _("  -r, --rvr=FILE             RVR recorded file to play (for debugging).\n"), stderr );
     lfputs( _("  -s, --showdrops            Print stats on frame drops (for debugging).\n"), stderr );
     lfputs( _("  -S, --saveoptions          Save command line options to the config file.\n"), stderr );
     lfputs( _("  -t, --xmltv=FILE           Read XMLTV listings from the given file.\n"), stderr );
+    lfputs( _("  -l, --xmltvlanguage=LANG   Use XMLTV data in given language, if available.\n"), stderr );
     lfputs( _("  -v, --verbose              Print debugging messages to stderr.\n"), stderr );
+    lfputs( _("  -X, --display=DISPLAY      Use the given X display to connect to.\n"), stderr );
     lfputs( _("  -x, --mixer=DEVICE[:CH]    The mixer device and channel to control.\n"
               "                             (defaults to /dev/mixer:line)\n\n"
               "                             Valid channels are:\n"
@@ -697,8 +633,6 @@ static void print_config_usage( char **argv )
     lfputs( _("  -b, --vbidevice=DEVICE     VBI device (defaults to /dev/vbi0).\n"), stderr );
     lfputs( _("  -c, --channel=CHANNEL      Tune to the specified channel on startup.\n"), stderr );
     lfputs( _("  -d, --device=DEVICE        video4linux device (defaults to /dev/video0).\n"), stderr );
-    lfputs( _("  -D, --driver=NAME          Output driver to use: Xv, DirectFB, mga,\n"
-              "                             matroxtv or SDL (defaults to Xv).\n"), stderr );
     lfputs( _("  -f, --frequencies=NAME     The frequency table to use for the tuner.\n"
               "                             (defaults to us-cable).\n\n"
               "                             Valid values are:\n"
@@ -717,7 +651,7 @@ static void print_config_usage( char **argv )
               "                                 custom (first run tvtime-scanner)\n\n"), stderr );
     lfputs( _("  -F, --configfile=FILE      Additional config file to load settings from.\n"), stderr );
     lfputs( _("  -h, --help                 Show this help message.\n"), stderr );
-    lfputs( _("  -H, --height=HEIGHT        Output window height (defaults to 576 pixels).\n"), stderr );
+    lfputs( _("  -g, --geometry=GEOMETRY    Sets the output window size.\n"), stderr );
     lfputs( _("  -i, --input=INPUTNUM       video4linux input number (defaults to 0).\n"), stderr );
     lfputs( _("  -I, --inputwidth=SAMPLING  Horizontal resolution of input\n"
               "                             (defaults to 720 pixels).\n"), stderr );
@@ -726,11 +660,9 @@ static void print_config_usage( char **argv )
     lfputs( _("  -n, --norm=NORM            The norm to use for the input.  tvtime supports:\n"
               "                             NTSC, NTSC-JP, SECAM, PAL, PAL-Nc, PAL-M,\n"
               "                             PAL-N or PAL-60 (defaults to NTSC).\n"), stderr );
-    lfputs( _("  -p, --fspos=POS            Set the fullscreen position: top, bottom or\n"
-              "                             centre (default).\n"), stderr );
     lfputs( _("  -R, --priority=PRI         Sets the process priority to run tvtime at.\n"), stderr );
     lfputs( _("  -t, --xmltv=FILE           Read XMLTV listings from the given file.\n"), stderr );
-    lfputs( _("  -X, --display=DISPLAY      Use the given X display to connect to.\n"), stderr );
+    lfputs( _("  -l, --xmltvlanguage=LANG   Use XMLTV data in given language, if available.\n"), stderr );
     lfputs( _("  -x, --mixer=DEVICE[:CH]    The mixer device and channel to control.\n"
               "                             (defaults to /dev/mixer:line)\n\n"
               "                             Valid channels are:\n"
@@ -763,29 +695,29 @@ config_t *config_new( void )
     }
 
     /* Default settings. */
-    ct->outputheight = 576;
+    ct->geometry = strdup( "0x576" );
     ct->verbose = 0;
     ct->aspect = 0;
     ct->debug = 0;
     ct->fullscreen = 0;
-    ct->priority = -19;
+    ct->alwaysontop = 0;
+    ct->priority = -10;
     ct->ntsc_mode = 0;
     ct->send_fields = 0;
-    ct->output_driver = strdup( "xv" );
-    ct->apply_luma_correction = 0;
-    ct->luma_correction = 1.0;
-    ct->useposition = 0;
     ct->fspos = 0;
-    ct->x = 320;
-    ct->y = 240;
     ct->picsaverestore = 1;
     ct->brightness = -1;
     ct->contrast = -1;
-    ct->colour = -1;
+    ct->saturation = -1;
     ct->hue = -1;
     ct->invert = 0;
     ct->cc = 0;
     ct->mirror = 0;
+    ct->boost = -1;
+    ct->quiet_screenshots = 0;
+    ct->unmute_volume = -1;
+    ct->muted = 0;
+    ct->mute_on_exit = 1;
 
     memset( ct->keymap, 0, sizeof( ct->keymap ) );
     memset( ct->keymap_arg, 0, sizeof( ct->keymap_arg ) );
@@ -809,14 +741,13 @@ config_t *config_new( void )
     }
     ct->audiomode = strdup( "stereo" );
     ct->xmltvfile = strdup( "none" );
+    ct->xmltvlanguage = strdup( "none" );
     ct->timeformat = strdup( "%X" );
     ct->channel_text_rgb = 0xffffff00; /* opaque yellow */
     ct->other_text_rgb = 0xfff5deb3;   /* opaque wheat */
 
     ct->uid = getuid();
 
-    ct->rvr_filename = 0;
-    ct->mpeg_filename = 0;
     ct->mixerdev = strdup( "/dev/mixer:line" );
 
     ct->deinterlace_method = strdup( "GreedyH" );
@@ -835,9 +766,6 @@ config_t *config_new( void )
     ct->config_filename = 0;
     ct->doc = 0;
 
-    ct->nummodes = 0;
-    ct->modelist = 0;
-
     /* Default key bindings. */
     ct->keymap[ 0 ] = TVTIME_NOCOMMAND;
     ct->keymap[ I_ESCAPE ] = TVTIME_QUIT;
@@ -859,29 +787,14 @@ config_t *config_new( void )
     ct->keymap[ I_F5 ] = TVTIME_PICTURE;
     ct->keymap[ I_F6 ] = TVTIME_PICTURE_DOWN;
     ct->keymap[ I_F7 ] = TVTIME_PICTURE_UP;
-    ct->keymap[ 'r' ] = TVTIME_CHANNEL_RENUMBER;
     ct->keymap[ 'd' ] = TVTIME_SHOW_STATS;
     ct->keymap[ 'f' ] = TVTIME_TOGGLE_FULLSCREEN;
     ct->keymap[ 'i' ] = TVTIME_TOGGLE_INPUT;
-    ct->keymap[ 'a' ] = TVTIME_TOGGLE_ASPECT;
     ct->keymap[ 's' ] = TVTIME_SCREENSHOT;
-    ct->keymap[ 't' ] = TVTIME_TOGGLE_DEINTERLACER;
-    ct->keymap[ 'p' ] = TVTIME_TOGGLE_PULLDOWN_DETECTION;
     ct->keymap[ ',' ] = TVTIME_MIXER_TOGGLE_MUTE;
-    ct->keymap[ '`' ] = TVTIME_TOGGLE_CONSOLE;
-    ct->keymap[ I_PGUP ] = TVTIME_SCROLL_CONSOLE_UP;
-    ct->keymap[ I_PGDN ] = TVTIME_SCROLL_CONSOLE_DOWN;
-    ct->keymap[ '=' ] = TVTIME_TOGGLE_FRAMERATE;
-    ct->keymap[ I_END ] = TVTIME_TOGGLE_PAUSE;
     ct->keymap[ 'e' ] = TVTIME_TOGGLE_AUDIO_MODE;
-    ct->keymap[ '<' ] = TVTIME_OVERSCAN_DOWN;
-    ct->keymap[ '>' ] = TVTIME_OVERSCAN_UP;
-    ct->keymap[ 'b' ] = TVTIME_TOGGLE_BARS;
-    ct->keymap[ '*' ] = TVTIME_TOGGLE_MODE;
     ct->keymap[ '/' ] = TVTIME_AUTO_ADJUST_WINDOW;
-    ct->keymap[ I_INSERT ] = TVTIME_TOGGLE_MATTE;
     ct->keymap[ 'v' ] = TVTIME_TOGGLE_ALWAYSONTOP;
-    ct->keymap[ 'o' ] = TVTIME_TOGGLE_COLOUR_INVERT;
     ct->keymap[ '0' ] = TVTIME_CHANNEL_0;
     ct->keymap[ '1' ] = TVTIME_CHANNEL_1;
     ct->keymap[ '2' ] = TVTIME_CHANNEL_2;
@@ -910,7 +823,7 @@ config_t *config_new( void )
     ct->keymapmenu[ 'q' ] = TVTIME_MENU_EXIT;
     ct->keymapmenu[ I_ESCAPE ] = TVTIME_MENU_EXIT;
     ct->buttonmapmenu[ 1 ] = TVTIME_MENU_ENTER;
-    ct->buttonmapmenu[ 3 ] = TVTIME_MENU_EXIT;
+    ct->buttonmapmenu[ 3 ] = TVTIME_MENU_ENTER;
     ct->buttonmapmenu[ 4 ] = TVTIME_MENU_UP;
     ct->buttonmapmenu[ 5 ] = TVTIME_MENU_DOWN;
 
@@ -925,7 +838,7 @@ config_t *config_new( void )
     /* First read in global settings. */
     asprintf( &base, "%s/tvtime.xml", CONFDIR );
     if( file_is_openable_for_read( base ) ) {
-        fprintf( stderr, _("Reading configuration from %s\n"), base );
+        lfprintf( stderr, _("Reading configuration from %s\n"), base );
         conf_xml_parse( ct, base );
     }
     free( base );
@@ -934,7 +847,7 @@ config_t *config_new( void )
     asprintf( &base, "%s/.tvtime/tvtime.xml", getenv( "HOME" ) );
     ct->config_filename = strdup( base );
     if( file_is_openable_for_read( base ) ) {
-        fprintf( stderr, _("Reading configuration from %s\n"), base );
+        lfprintf( stderr, _("Reading configuration from %s\n"), base );
         conf_xml_parse( ct, base );
     }
     free( base );
@@ -947,7 +860,7 @@ int config_parse_tvtime_command_line( config_t *ct, int argc, char **argv )
     static struct option long_options[] = {
         { "help", 0, 0, 'h' },
         { "verbose", 0, 0, 'v' },
-        { "height", 1, 0, 'H' },
+        { "geometry", 1, 0, 'g' },
         { "saveoptions", 0, 0, 'S' },
         { "inputwidth", 1, 0, 'I' },
         { "driver", 1, 0, 'D' },
@@ -964,20 +877,19 @@ int config_parse_tvtime_command_line( config_t *ct, int argc, char **argv )
         { "window", 0, 0, 'M' },
         { "slave", 0, 0, 'k' },
         { "widescreen", 0, 0, 'a' },
-        { "rvr", 1, 0, 'r' },
-        { "mpeg", 1, 0, 'g' },
-        { "fspos", 1, 0, 'p' },
         { "xmltv", 1, 0, 't' },
+        { "xmltvlanguage", 1, 0, 'l' },
         { "display", 1, 0, 'X' },
         { "version", 0, 0, 'Q' },
         { 0, 0, 0, 0 }
     };
     int option_index = 0;
     int saveoptions = 0;
+    int filename_specified = 0;
     char c;
 
     if( argc ) {
-        while( (c = getopt_long( argc, argv, "ahkmMsSvF:r:H:I:d:b:i:c:n:D:f:x:p:X:t:Qg:",
+        while( (c = getopt_long( argc, argv, "ahkmMsSvF:r:g:I:d:b:i:c:n:D:f:x:X:t:l:Qg:",
                 long_options, &option_index )) != -1 ) {
             switch( c ) {
             case 'a': ct->aspect = 1; break;
@@ -989,7 +901,10 @@ int config_parse_tvtime_command_line( config_t *ct, int argc, char **argv )
             case 'v': ct->verbose = 1; break;
             case 't': if( ct->xmltvfile ) { free( ct->xmltvfile ); }
                       ct->xmltvfile = expand_user_path( optarg ); break;
+            case 'l': if( ct->xmltvlanguage ) { free( ct->xmltvlanguage ); }
+                      ct->xmltvlanguage = strdup( optarg ); break;
             case 'F': if( ct->config_filename ) free( ct->config_filename );
+                      filename_specified = 1;
                       ct->config_filename = expand_user_path( optarg );
                       if( ct->config_filename ) {
                           lfprintf
@@ -999,19 +914,11 @@ int config_parse_tvtime_command_line( config_t *ct, int argc, char **argv )
                           conf_xml_parse( ct, ct->config_filename );
                       }
                       break;
-            case 'r': if( ct->rvr_filename ) { free( ct->rvr_filename ); }
-                      ct->rvr_filename = strdup( optarg ); break;
-            case 'g': if( ct->mpeg_filename ) { free( ct->mpeg_filename ); }
-                      ct->mpeg_filename = strdup( optarg ); break;
             case 'x': if( ct->mixerdev ) { free( ct->mixerdev ); }
                       ct->mixerdev = strdup( optarg ); break;
             case 'X': setenv( "DISPLAY", optarg, 1 ); break;
-            case 'H': if( tolower( optarg[ 0 ] ) == 'f' ) {
-                          ct->outputheight = -1;
-                      } else {
-                          ct->outputheight = atoi( optarg );
-                      }
-                      break;
+            case 'g': if( ct->geometry ) { free( ct->geometry ); }
+                      ct->geometry = strdup( optarg ); break;
             case 'I': ct->inputwidth = atoi( optarg ); break;
             case 'd': free( ct->v4ldev ); ct->v4ldev = strdup( optarg ); break;
             case 'b': free( ct->vbidev ); ct->vbidev = strdup( optarg ); break;
@@ -1019,16 +926,6 @@ int config_parse_tvtime_command_line( config_t *ct, int argc, char **argv )
             case 'c': ct->prev_channel = ct->start_channel;
                       ct->start_channel = atoi( optarg ); break;
             case 'n': free( ct->norm ); ct->norm = strdup( optarg ); break;
-            case 'D': if( ct->output_driver ) { free( ct->output_driver ); }
-                      ct->output_driver = strdup( optarg ); break;
-            case 'p': if( tolower( optarg[ 0 ] ) == 't' ) {
-                          ct->fspos = 1;
-                      } else if( tolower( optarg[ 0 ] ) == 'b' ) {
-                          ct->fspos = 2;
-                      } else {
-                          ct->fspos = 0;
-                      }
-                      break;
             case 'f': free( ct->freq ); ct->freq = strdup( optarg ); break;
             case 'Q': print_copyright(); return 0;
             default:
@@ -1038,6 +935,25 @@ int config_parse_tvtime_command_line( config_t *ct, int argc, char **argv )
         }
     }
 
+    if( !filename_specified ) {
+        char *fifofile = get_tvtime_fifo_filename( config_get_uid( ct ) );
+        int fifofd;
+
+        if( !fifofile ) {
+            lfprintf( stderr, _("%s: Cannot allocate memory.\n"), argv[ 0 ] );
+            return 0;
+        }
+
+        fifofd = open( fifofile, O_WRONLY | O_NONBLOCK );
+        if( fifofd >= 0 ) {
+            lfprintf( stderr,
+                      _("Cannot run two instances of tvtime with the same configuration.\n") );
+            return 0;
+        }
+        close( fifofd );
+    }
+
+
     ct->doc = configsave_open( ct->config_filename );
 
     if( ct->doc && saveoptions ) {
@@ -1058,18 +974,7 @@ int config_parse_tvtime_command_line( config_t *ct, int argc, char **argv )
         snprintf( tempstring, sizeof( tempstring ), "%d", ct->verbose );
         config_save( ct, "Verbose", tempstring );
 
-        config_save( ct, "OutputDriver", ct->output_driver );
-
-        if( ct->fspos == 0 ) {
-            config_save( ct, "FullscreenPosition", "Centre" );
-        } else if( ct->fspos == 1 ) {
-            config_save( ct, "FullscreenPosition", "Top" );
-        } else if( ct->fspos == 2 ) {
-            config_save( ct, "FullscreenPosition", "Bottom" );
-        }
-
-        snprintf( tempstring, sizeof( tempstring ), "%d", ct->outputheight );
-        config_save( ct, "OutputHeight", tempstring );
+        config_save( ct, "WindowGeometry", ct->geometry );
 
         snprintf( tempstring, sizeof( tempstring ), "%d", ct->inputwidth );
         config_save( ct, "InputWidth", tempstring );
@@ -1086,6 +991,7 @@ int config_parse_tvtime_command_line( config_t *ct, int argc, char **argv )
         config_save( ct, "MixerDevice", ct->mixerdev );
 
         config_save( ct, "XMLTVFile", ct->xmltvfile );
+        config_save( ct, "XMLTVLanguage", ct->xmltvlanguage );
     }
 
     return 1;
@@ -1095,7 +1001,7 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
 {
     static struct option long_options[] = {
         { "help", 0, 0, 'h' },
-        { "height", 1, 0, 'H' },
+        { "geometry", 1, 0, 'g' },
         { "inputwidth", 1, 0, 'I' },
         { "driver", 1, 0, 'D' },
         { "input", 2, 0, 'i' },
@@ -1109,12 +1015,13 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
         { "fullscreen", 0, 0, 'm' },
         { "window", 0, 0, 'M' },
         { "widescreen", 0, 0, 'a' },
-        { "fspos", 1, 0, 'p' },
         { "xmltv", 2, 0, 't' },
+        { "xmltvlanguage", 2, 0, 'l' },
         { "priority", 2, 0, 'R' },
         { 0, 0, 0, 0 }
     };
     int option_index = 0;
+    int filename_specified = 0;
     char c;
 
     if( argc == 1 ) {
@@ -1122,13 +1029,14 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
         return 0;
     }
 
-    while( (c = getopt_long( argc, argv, "ahmMF:H:I:d:b:i:c:n:D:f:x:p:t:R:",
+    while( (c = getopt_long( argc, argv, "ahmMF:g:I:d:b:i:c:n:D:f:x:t:l:R:",
             long_options, &option_index )) != -1 ) {
         switch( c ) {
         case 'a': ct->aspect = 1; break;
         case 'm': ct->fullscreen = 1; break;
         case 'M': ct->fullscreen = 0; break;
         case 'F': if( ct->config_filename ) free( ct->config_filename );
+                  filename_specified = 1;
                   ct->config_filename = expand_user_path( optarg );
                   if( ct->config_filename ) {
                       lfprintf( stderr,
@@ -1139,12 +1047,8 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
                   break;
         case 'x': if( ct->mixerdev ) { free( ct->mixerdev ); }
                   ct->mixerdev = strdup( optarg ); break;
-        case 'H': if( tolower( optarg[ 0 ] ) == 'f' ) {
-                      ct->outputheight = -1;
-                  } else {
-                      ct->outputheight = atoi( optarg );
-                  }
-                  break;
+        case 'g': if( ct->geometry ) { free( ct->geometry ); }
+                  ct->geometry = strdup( optarg ); break;
         case 'I': ct->inputwidth = atoi( optarg ); break;
         case 'd': if( !optarg ) {
                       fprintf( stdout, "V4LDevice:%s\n",
@@ -1179,6 +1083,14 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
                       ct->xmltvfile = expand_user_path( optarg );
                   }
                   break;
+        case 'l': if( !optarg ) {
+                      fprintf( stdout, "XMLTVLanguage:%s\n",
+                               config_get_xmltv_language( ct ) );
+                  } else {
+                      if( ct->xmltvlanguage ) free( ct->xmltvlanguage );
+                      ct->xmltvlanguage = strdup( optarg );
+                  }
+                  break;
         case 'n': if( !optarg ) {
                       fprintf( stdout, "Norm:%s\n", config_get_v4l_norm( ct ) );
                   } else {
@@ -1186,16 +1098,6 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
                       ct->norm = strdup( optarg );
                   }
                   break;
-        case 'D': if( ct->output_driver ) { free( ct->output_driver ); }
-                  ct->output_driver = strdup( optarg ); break;
-        case 'p': if( tolower( optarg[ 0 ] ) == 't' ) {
-                      ct->fspos = 1;
-                  } else if( tolower( optarg[ 0 ] ) == 'b' ) {
-                      ct->fspos = 2;
-                  } else {
-                      ct->fspos = 0;
-                  }
-                  break;
         case 'f': if( !optarg ) {
                       fprintf( stdout, "Frequencies:%s\n",
                                config_get_v4l_freq( ct ) );
@@ -1217,6 +1119,24 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
         }
     }
 
+    if( !filename_specified ) {
+        char *fifofile = get_tvtime_fifo_filename( config_get_uid( ct ) );
+        int fifofd;
+
+        if( !fifofile ) {
+            lfprintf( stderr, _("%s: Cannot allocate memory.\n"), argv[ 0 ] );
+            return 0;
+        }
+
+        fifofd = open( fifofile, O_WRONLY | O_NONBLOCK );
+        if( fifofd >= 0 ) {
+            lfprintf( stderr,
+                      _("Cannot update configuration while tvtime running.\n") );
+            return 0;
+        }
+        close( fifofd );
+    }
+
     ct->doc = configsave_open( ct->config_filename );
 
     if( ct->doc ) {
@@ -1236,18 +1156,7 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
         snprintf( tempstring, sizeof( tempstring ), "%d", ct->verbose );
         config_save( ct, "Verbose", tempstring );
 
-        config_save( ct, "OutputDriver", ct->output_driver );
-
-        if( ct->fspos == 0 ) {
-            config_save( ct, "FullscreenPosition", "Centre" );
-        } else if( ct->fspos == 1 ) {
-            config_save( ct, "FullscreenPosition", "Top" );
-        } else if( ct->fspos == 2 ) {
-            config_save( ct, "FullscreenPosition", "Bottom" );
-        }
-
-        snprintf( tempstring, sizeof( tempstring ), "%d", ct->outputheight );
-        config_save( ct, "OutputHeight", tempstring );
+        config_save( ct, "WindowGeometry", ct->geometry );
 
         snprintf( tempstring, sizeof( tempstring ), "%d", ct->inputwidth );
         config_save( ct, "InputWidth", tempstring );
@@ -1265,6 +1174,7 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
         config_save( ct, "MixerDevice", ct->mixerdev );
 
         config_save( ct, "XMLTVFile", ct->xmltvfile );
+        config_save( ct, "XMLTVLanguage", ct->xmltvlanguage );
 
         snprintf( tempstring, sizeof( tempstring ), "%d", ct->priority );
         config_save( ct, "ProcessPriority", tempstring );
@@ -1317,16 +1227,15 @@ int config_parse_tvtime_scanner_command_line( config_t *ct, int argc,
 void config_free_data( config_t *ct )
 {
     if( ct->doc ) xmlFreeDoc( ct->doc );
+    if( ct->geometry ) free( ct->geometry );
     if( ct->v4ldev ) free( ct->v4ldev );
     if( ct->norm ) free( ct->norm );
     if( ct->freq ) free( ct->freq );
     if( ct->ssdir ) free( ct->ssdir );
     if( ct->audiomode ) free( ct->audiomode );
     if( ct->xmltvfile ) free( ct->xmltvfile );
+    if( ct->xmltvlanguage ) free( ct->xmltvlanguage );
     if( ct->timeformat ) free( ct->timeformat );
-    if( ct->output_driver ) free( ct->output_driver );
-    if( ct->rvr_filename ) free( ct->rvr_filename );
-    if( ct->mpeg_filename ) free( ct->mpeg_filename );
     if( ct->mixerdev ) free( ct->mixerdev );
     if( ct->vbidev ) free( ct->vbidev );
     if( ct->config_filename ) free( ct->config_filename );
@@ -1337,14 +1246,6 @@ void config_delete( config_t *ct )
 {
     int i;
 
-    while( ct->modelist ) {
-        tvtime_modelist_t *mode = ct->modelist;
-        ct->modelist = mode->next;
-        config_free_data( &(mode->settings) );
-        free( mode->name );
-        free( mode );
-    }
-
     for( i = 0; i < 8 * MAX_KEYSYMS; i++ ) {
         if( ct->keymap_arg[ i ] ) free( ct->keymap_arg[ i ] );
         if( ct->keymapmenu_arg[ i ] ) free( ct->keymapmenu_arg[ i ] );
@@ -1499,28 +1400,6 @@ const char *config_button_to_menu_command_argument( config_t *ct, int button )
     }
 }
 
-int config_get_num_modes( config_t *ct )
-{
-    return ct->nummodes;
-}
-
-config_t *config_get_mode_info( config_t *ct, int mode )
-{
-    tvtime_modelist_t *cur = ct->modelist;
-
-    if( !cur ) {
-        /* No modes. */
-        return 0;
-    }
-
-    while( mode && cur->next ) {
-        cur = cur->next;
-        mode--;
-    }
-
-    return &(cur->settings);
-}
-
 int config_get_verbose( config_t *ct )
 {
     return ct->verbose;
@@ -1531,34 +1410,14 @@ int config_get_send_fields( config_t *ct )
     return ct->send_fields;
 }
 
-const char *config_get_output_driver( config_t *ct )
-{
-    return ct->output_driver;
-}
-
 int config_get_debug( config_t *ct )
 {
     return ct->debug;
 }
 
-int config_get_outputheight( config_t *ct )
+const char *config_get_geometry( config_t *ct )
 {
-    return ct->outputheight;
-}
-
-int config_get_useposition( config_t *ct )
-{
-    return ct->useposition;
-}
-
-int config_get_output_x( config_t *ct )
-{
-    return ct->x;
-}
-
-int config_get_output_y( config_t *ct )
-{
-    return ct->y;
+    return ct->geometry;
 }
 
 int config_get_inputwidth( config_t *ct )
@@ -1586,21 +1445,11 @@ int config_get_inputnum( config_t *ct )
     return ct->inputnum;
 }
 
-int config_get_apply_luma_correction( config_t *ct )
-{
-    return ct->apply_luma_correction;
-}
-
 const char *config_get_deinterlace_method( config_t *ct )
 {
     return ct->deinterlace_method;
 }
 
-double config_get_luma_correction( config_t *ct )
-{
-    return ct->luma_correction;
-}
-
 const char *config_get_v4l_device( config_t *ct )
 {
     return ct->v4ldev;
@@ -1616,6 +1465,11 @@ int config_get_fullscreen( config_t *ct )
     return ct->fullscreen;
 }
 
+int config_get_alwaysontop( config_t *ct )
+{
+    return ct->alwaysontop;
+}
+
 int config_get_priority( config_t *ct )
 {
     return ct->priority;
@@ -1676,16 +1530,6 @@ const char *config_get_screenshot_dir( config_t *ct )
     return ct->ssdir;
 }
 
-const char *config_get_rvr_filename( config_t *ct )
-{
-    return ct->rvr_filename;
-}
-
-const char *config_get_mpeg_filename( config_t *ct )
-{
-    return ct->mpeg_filename;
-}
-
 int config_get_framerate_mode( config_t *ct )
 {
     return ct->framerate;
@@ -1721,9 +1565,9 @@ int config_get_global_contrast( config_t *ct )
     return ct->contrast;
 }
 
-int config_get_global_colour( config_t *ct )
+int config_get_global_saturation( config_t *ct )
 {
-    return ct->colour;
+    return ct->saturation;
 }
 
 int config_get_global_hue( config_t *ct )
@@ -1741,6 +1585,11 @@ const char *config_get_xmltv_file( config_t *ct )
     return ct->xmltvfile;
 }
 
+const char *config_get_xmltv_language( config_t *ct )
+{
+    return ct->xmltvlanguage;
+}
+
 int config_get_invert( config_t *ct )
 {
     return ct->invert;
@@ -1756,3 +1605,28 @@ int config_get_mirror( config_t *ct )
     return ct->mirror;
 }
 
+int config_get_audio_boost( config_t *ct )
+{
+    return ct->boost;
+}
+
+int config_get_quiet_screenshots( config_t *ct )
+{
+    return ct->quiet_screenshots;
+}
+
+int config_get_unmute_volume( config_t *ct )
+{
+    return ct->unmute_volume;
+}
+
+int config_get_muted( config_t *ct )
+{
+    return ct->muted;
+}
+
+int config_get_mute_on_exit( config_t *ct )
+{
+    return ct->mute_on_exit;
+}
+
