@@ -49,6 +49,7 @@ ings in this Software without prior written authorization from him.
 #include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
+#include <sys/param.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -142,17 +143,30 @@ int mkdir_and_force_owner( const char *path, uid_t uid, gid_t gid )
 
 char *get_tvtime_fifo_filename( uid_t uid )
 {
-    /* SUSv2 guarantees that 'Host names are limited to 255  bytes'. */
-    static char hostname[ 256 ];
+    /* SUSv3 provides no guarantees to the length of a hostname.  It
+     * specifies that HOST_NAME_MAX will give you the maximum number
+     * of bytes, however, this is not defined on all platforms.
+     */
+#ifdef HOST_NAME_MAX
+    size_t hostname_size = HOST_NAME_MAX;
+#else
+#ifdef MAXHOSTNAMELEN
+    /* MAXHOSTNAMELEN is widely used on UNIX systems, but was never
+     * formalised by POSIX.  Strangely enough, POSIX didn't adopt this
+     * symbol in SUSv3.
+     */
+    size_t hostname_size = MAXHOSTNAMELEN;
+#else
+    size_t hostname_size = 255;
+#endif /* MAXHOSTNAMELEN */
+#endif /* HOST_NAME_MAX */
+    char *hostname = 0;
+    char *hostname_realloc = 0;
+
     struct passwd *pwuid = 0;
     char *fifodir;
     char *fifo;
 
-    if( gethostname( hostname, sizeof( hostname ) ) == -1 ) {
-        /* Put errno in the filename for interest. */
-        snprintf( hostname, sizeof( hostname ), "unknown%d", errno );
-    }
-
     /* Create string for the directory in FIFODIR */
     pwuid = getpwuid( uid );
     if( pwuid ) {
@@ -165,6 +179,29 @@ char *get_tvtime_fifo_filename( uid_t uid )
         }
     }
 
+    /* Try to get a hostname */
+    hostname = malloc( hostname_size * sizeof( char ) );
+    if (hostname == 0)
+        return 0;
+    errno = 0;
+    while( gethostname( hostname, hostname_size ) < 0 ) {
+        if( errno == ENAMETOOLONG ) {
+            hostname_size *= 2;
+            hostname_realloc = realloc( hostname, hostname_size );
+            if( !hostname_realloc ) {
+                /* Use a partial hostname. */
+                hostname_size /= 2;
+                break;
+            } else {
+                hostname = hostname_realloc;
+            }
+        } else {
+            /* Unknown error.  Put errno in the filename for interest. */
+            snprintf( hostname, hostname_size, "unknown%d", errno );
+            break;
+        }
+    }
+
     /* If we can't use our /tmp directory, put the fifo in $HOME. */
     if( !mkdir_and_force_owner( fifodir, uid, getgid() ) ) {
         if( asprintf( &fifo, "%s/.tvtime/tvtimefifo-%s",
@@ -176,6 +213,7 @@ char *get_tvtime_fifo_filename( uid_t uid )
             fifo = 0;
         }
     }
+    free( hostname );
     free( fifodir );
     return fifo;
 }
@@ -412,11 +450,11 @@ static command_names_t command_table[] = {
     { "CHANNEL_SKIP", TVTIME_CHANNEL_SKIP },
     { "CHANNEL_UP", TVTIME_CHANNEL_INC },
 
-    { "COLOR_DOWN", TVTIME_COLOUR_DOWN },
-    { "COLOR_UP", TVTIME_COLOUR_UP },
+    { "COLOR_DOWN", TVTIME_SATURATION_DOWN },
+    { "COLOR_UP", TVTIME_SATURATION_UP },
 
-    { "COLOUR_DOWN", TVTIME_COLOUR_DOWN },
-    { "COLOUR_UP", TVTIME_COLOUR_UP },
+    { "COLOUR_DOWN", TVTIME_SATURATION_DOWN },
+    { "COLOUR_UP", TVTIME_SATURATION_UP },
 
     { "CONTRAST_DOWN", TVTIME_CONTRAST_DOWN },
     { "CONTRAST_UP", TVTIME_CONTRAST_UP },
@@ -438,9 +476,6 @@ static command_names_t command_table[] = {
 
     { "LEFT", TVTIME_LEFT },
 
-    { "LUMA_DOWN", TVTIME_LUMA_DOWN },
-    { "LUMA_UP", TVTIME_LUMA_UP },
-
     { "MENU_DOWN", TVTIME_MENU_DOWN },
     { "MENU_ENTER", TVTIME_MENU_ENTER },
     { "MENU_EXIT", TVTIME_MENU_EXIT },
@@ -464,21 +499,26 @@ static command_names_t command_table[] = {
     { "RIGHT", TVTIME_RIGHT },
     { "RUN_COMMAND", TVTIME_RUN_COMMAND },
 
+    { "SATURATION_DOWN", TVTIME_SATURATION_DOWN },
+    { "SATURATION_UP", TVTIME_SATURATION_UP },
+
     { "SAVE_PICTURE_GLOBAL", TVTIME_SAVE_PICTURE_GLOBAL },
     { "SAVE_PICTURE_CHANNEL", TVTIME_SAVE_PICTURE_CHANNEL },
 
     { "SCREENSHOT", TVTIME_SCREENSHOT },
-    { "SCROLL_CONSOLE_DOWN", TVTIME_SCROLL_CONSOLE_DOWN },
-    { "SCROLL_CONSOLE_UP", TVTIME_SCROLL_CONSOLE_UP },
 
+    { "SET_AUDIO_BOOST", TVTIME_SET_AUDIO_BOOST },
     { "SET_AUDIO_MODE", TVTIME_SET_AUDIO_MODE },
     { "SET_DEINTERLACER", TVTIME_SET_DEINTERLACER },
     { "SET_FRAMERATE", TVTIME_SET_FRAMERATE },
     { "SET_FREQUENCY_TABLE", TVTIME_SET_FREQUENCY_TABLE },
     { "SET_FULLSCREEN_POSITION", TVTIME_SET_FULLSCREEN_POSITION },
+    { "SET_INPUT", TVTIME_SET_INPUT },
     { "SET_INPUT_WIDTH", TVTIME_SET_INPUT_WIDTH },
     { "SET_MATTE", TVTIME_SET_MATTE },
     { "SET_NORM", TVTIME_SET_NORM },
+    { "SET_STATION", TVTIME_SET_STATION },
+    { "SET_XMLTV_LANGUAGE", TVTIME_SET_XMLTV_LANGUAGE },
 
     { "SHOW_DEINTERLACER_INFO", TVTIME_SHOW_DEINTERLACER_INFO },
     { "SHOW_MENU", TVTIME_SHOW_MENU },
@@ -491,26 +531,26 @@ static command_names_t command_table[] = {
     { "TOGGLE_AUDIO_MODE", TVTIME_TOGGLE_AUDIO_MODE },
     { "TOGGLE_BARS", TVTIME_TOGGLE_BARS },
     { "TOGGLE_CC", TVTIME_TOGGLE_CC },
+    { "TOGGLE_CHANNEL_PAL_DK", TVTIME_TOGGLE_CHANNEL_PAL_DK },
     { "TOGGLE_CHROMA_KILL", TVTIME_TOGGLE_CHROMA_KILL },
     { "TOGGLE_COLOR_INVERT", TVTIME_TOGGLE_COLOUR_INVERT },
     { "TOGGLE_COLOUR_INVERT", TVTIME_TOGGLE_COLOUR_INVERT },
-    { "TOGGLE_CONSOLE", TVTIME_TOGGLE_CONSOLE },
-    /* { "TOGGLE_CREDITS", TVTIME_TOGGLE_CREDITS }, Disabled for 0.9.8 */
     { "TOGGLE_DEINTERLACER", TVTIME_TOGGLE_DEINTERLACER },
     { "TOGGLE_FULLSCREEN", TVTIME_TOGGLE_FULLSCREEN },
     { "TOGGLE_FRAMERATE", TVTIME_TOGGLE_FRAMERATE },
     { "TOGGLE_INPUT", TVTIME_TOGGLE_INPUT },
-    { "TOGGLE_LUMA_CORRECTION", TVTIME_TOGGLE_LUMA_CORRECTION },
     { "TOGGLE_MATTE", TVTIME_TOGGLE_MATTE },
     { "TOGGLE_MIRROR", TVTIME_TOGGLE_MIRROR },
-    { "TOGGLE_MODE", TVTIME_TOGGLE_MODE },
     { "TOGGLE_MUTE", TVTIME_TOGGLE_MUTE },
     { "TOGGLE_NTSC_CABLE_MODE", TVTIME_TOGGLE_NTSC_CABLE_MODE },
+    { "TOGGLE_PAL_DK_AUDIO", TVTIME_TOGGLE_PAL_DK_AUDIO },
     { "TOGGLE_PAL_SECAM", TVTIME_TOGGLE_PAL_SECAM },
     { "TOGGLE_PAUSE", TVTIME_TOGGLE_PAUSE },
     { "TOGGLE_PULLDOWN_DETECTION", TVTIME_TOGGLE_PULLDOWN_DETECTION },
+    { "TOGGLE_QUIET_SCREENSHOTS", TVTIME_TOGGLE_QUIET_SCREENSHOTS },
     { "TOGGLE_SIGNAL_DETECTION", TVTIME_TOGGLE_SIGNAL_DETECTION },
     { "TOGGLE_XDS", TVTIME_TOGGLE_XDS },
+    { "TOGGLE_XMLTV_LANGUAGE", TVTIME_TOGGLE_XMLTV_LANGUAGE },
 
     { "UP", TVTIME_UP },
 
@@ -572,7 +612,10 @@ int tvtime_command_takes_arguments( int command )
             command == TVTIME_SHOW_MENU || command == TVTIME_SET_FRAMERATE ||
             command == TVTIME_SET_AUDIO_MODE || command == TVTIME_SET_INPUT_WIDTH ||
             command == TVTIME_SET_MATTE || command == TVTIME_SET_FREQUENCY_TABLE ||
-            command == TVTIME_RUN_COMMAND || command == TVTIME_SET_FULLSCREEN_POSITION);
+            command == TVTIME_RUN_COMMAND || command == TVTIME_SET_FULLSCREEN_POSITION ||
+            command == TVTIME_MIXER_UP || command == TVTIME_MIXER_DOWN ||
+            command == TVTIME_SET_AUDIO_BOOST || command == TVTIME_SET_STATION ||
+            command == TVTIME_SET_XMLTV_LANGUAGE || command == TVTIME_SET_INPUT);
 }
 
 #ifdef ENABLE_NLS
