@@ -188,7 +188,10 @@ struct videoinput_s
 
     int width;
     int height;
+    int maxwidth;
     int norm;
+    int volume;
+    int dkmode;
 
     int isbttv;
     int isv4l2;
@@ -374,8 +377,11 @@ uint8_t *videoinput_next_frame( videoinput_t *vidin, int *frameid )
  
         cur_buf.type = vidin->capbuffers[ 0 ].vidbuf.type;
         if( ioctl( vidin->grab_fd, VIDIOC_DQBUF, &cur_buf ) < 0 ) {
-            fprintf( stderr, "videoinput: Can't read frame. Error was: %s (%d).\n",
-                     strerror( errno ), vidin->frames_since_start );
+            /* some drivers return EIO when there is no signal */
+            if( errno != EIO ) {
+                fprintf( stderr, "videoinput: Can't read frame. Error was: %s (%d).\n",
+                         strerror( errno ), vidin->frames_since_start );
+            }
             /* We must now restart capture. */
             videoinput_stop_capture_v4l2( vidin );
             videoinput_free_all_frames( vidin );
@@ -418,7 +424,7 @@ int videoinput_buffer_invalid( videoinput_t *vidin, int frameid )
 }
 
 videoinput_t *videoinput_new( const char *v4l_device, int capwidth,
-                              int norm, int verbose )
+                              int volume, int norm, int verbose )
 {
     videoinput_t *vidin = malloc( sizeof( videoinput_t ) );
     struct video_capability caps_v4l1;
@@ -445,6 +451,8 @@ videoinput_t *videoinput_new( const char *v4l_device, int capwidth,
     vidin->curframe = 0;
     vidin->verbose = verbose;
     vidin->norm = norm;
+    vidin->volume = volume;
+    vidin->dkmode = 0;
     vidin->height = videoinput_get_norm_height( norm );
     vidin->cur_tuner_state = TUNER_STATE_NO_SIGNAL;
 
@@ -657,11 +665,28 @@ videoinput_t *videoinput_new( const char *v4l_device, int capwidth,
 
         if( capwidth != imgformat.fmt.pix.width && verbose ) {
             fprintf( stderr, "videoinput: Width %d too high, using %d "
-                             "instead as suggested by the driver.",
+                             "instead as suggested by the driver.\n",
                      capwidth, imgformat.fmt.pix.width );
         }
         vidin->width = imgformat.fmt.pix.width;
 
+        /* Query the maximum input width - I wish there was a nicer way */
+        imgformat.fmt.pix.width = 10000;
+        if( ioctl( vidin->grab_fd, VIDIOC_TRY_FMT, &imgformat ) < 0 ) {
+            vidin->maxwidth = 768;
+            if( verbose ) {
+                fprintf( stderr, "videoinput: Your capture card driver does "
+                                 "not implement the TRY_FMT query.\n"
+                                 "Setting maximum input width to 768.\n" );
+            }
+        } else {
+            vidin->maxwidth = imgformat.fmt.pix.width;
+            if( verbose ) {
+                fprintf( stderr, "videoinput: Maximum input width: %d "
+                                 "pixels.\n", vidin->maxwidth );
+            }
+        }
+
     } else {
         if( ioctl( vidin->grab_fd, VIDIOCGCAP, &caps_v4l1 ) < 0 ) {
             fprintf( stderr, "videoinput: video4linux device '%s' refuses "
@@ -671,10 +696,15 @@ videoinput_t *videoinput_new( const char *v4l_device, int capwidth,
             free( vidin );
             return 0;
         }
+        vidin->maxwidth = caps_v4l1.maxwidth;
+        if( verbose ) {
+            fprintf( stderr, "videoinput: Maximum input width: %d pixels.\n",
+                     vidin->maxwidth );
+        }
         if( capwidth > caps_v4l1.maxwidth ) {
             if( verbose ) {
                 fprintf( stderr, "videoinput: Width %d too high, using %d "
-                                 "instead as suggested by the driver.",
+                                 "instead as suggested by the driver.\n",
                          capwidth, caps_v4l1.maxwidth );
             }
             capwidth = caps_v4l1.maxwidth;
@@ -682,7 +712,7 @@ videoinput_t *videoinput_new( const char *v4l_device, int capwidth,
         if( capwidth < caps_v4l1.minwidth ) {
             if( verbose ) {
                 fprintf( stderr, "videoinput: Width %d too low, using %d "
-                                 "instead as suggested by the driver.",
+                                 "instead as suggested by the driver.\n",
                          capwidth, caps_v4l1.minwidth );
             }
             capwidth = caps_v4l1.minwidth;
@@ -969,7 +999,10 @@ void videoinput_set_hue( videoinput_t *vidin, int newhue )
 
 void videoinput_set_hue_relative( videoinput_t *vidin, int offset )
 {
-    videoinput_set_hue( vidin, videoinput_get_hue( vidin ) + offset );
+    int hue = videoinput_get_hue( vidin );
+    if( hue + offset >= 0 ) {
+        videoinput_set_hue( vidin, hue + offset );
+    }
 }
 
 int videoinput_get_brightness( videoinput_t *vidin )
@@ -1006,7 +1039,10 @@ void videoinput_set_brightness( videoinput_t *vidin, int newbright )
 
 void videoinput_set_brightness_relative( videoinput_t *vidin, int offset )
 {
-    videoinput_set_brightness( vidin, videoinput_get_brightness( vidin ) + offset );
+    int brightness = videoinput_get_brightness( vidin );
+    if( brightness + offset >= 0 ) {
+        videoinput_set_brightness( vidin, brightness + offset );
+    }
 }
 
 int videoinput_get_contrast( videoinput_t *vidin )
@@ -1043,10 +1079,13 @@ void videoinput_set_contrast( videoinput_t *vidin, int newcont )
 
 void videoinput_set_contrast_relative( videoinput_t *vidin, int offset )
 {
-    videoinput_set_contrast( vidin, videoinput_get_contrast( vidin ) + offset );
+    int contrast = videoinput_get_contrast( vidin );
+    if( contrast + offset >= 0 ) {
+        videoinput_set_contrast( vidin, contrast + offset );
+    }
 }
 
-int videoinput_get_colour( videoinput_t *vidin )
+int videoinput_get_saturation( videoinput_t *vidin )
 {
     if( vidin->isv4l2 ) {
         return (int) ((videoinput_get_control_v4l2( vidin, V4L2_CID_SATURATION ) * 100.0) + 0.5);
@@ -1061,26 +1100,29 @@ int videoinput_get_colour( videoinput_t *vidin )
     return 0;
 }
 
-void videoinput_set_colour( videoinput_t *vidin, int newcolour )
+void videoinput_set_saturation( videoinput_t *vidin, int newsaturation )
 {
-    if( newcolour > 100 ) newcolour = 100;
-    if( newcolour <   0 ) newcolour = 50;
+    if( newsaturation > 100 ) newsaturation = 100;
+    if( newsaturation <   0 ) newsaturation = 50;
 
     if( vidin->isv4l2 ) {
-        videoinput_set_control_v4l2( vidin, V4L2_CID_SATURATION, ((double) newcolour) / 100.0 );
+        videoinput_set_control_v4l2( vidin, V4L2_CID_SATURATION, ((double) newsaturation) / 100.0 );
     } else {
         struct video_picture grab_pict;
 
         if( ioctl( vidin->grab_fd, VIDIOCGPICT, &grab_pict ) >= 0 ) {
-            grab_pict.colour = (int) (((((double) newcolour) / 100.0) * 65535.0) + 0.5);
+            grab_pict.colour = (int) (((((double) newsaturation) / 100.0) * 65535.0) + 0.5);
             ioctl( vidin->grab_fd, VIDIOCSPICT, &grab_pict );
         }
     }
 }
 
-void videoinput_set_colour_relative( videoinput_t *vidin, int offset )
+void videoinput_set_saturation_relative( videoinput_t *vidin, int offset )
 {
-    videoinput_set_colour( vidin, videoinput_get_colour( vidin ) + offset );
+    int saturation = videoinput_get_saturation( vidin );
+    if( saturation + offset >= 0 ) {
+        videoinput_set_saturation( vidin, saturation + offset );
+    }
 }
 
 static void videoinput_do_mute( videoinput_t *vidin, int mute )
@@ -1098,8 +1140,8 @@ static void videoinput_do_mute( videoinput_t *vidin, int mute )
                 fprintf( stderr, "videoinput: Include this error: '%s'\n", strerror( errno ) );
             }
 
-            if( !mute ) {
-                videoinput_set_control_v4l2( vidin, V4L2_CID_AUDIO_VOLUME, 0.9155 );
+            if( !mute && vidin->volume > 0 ) {
+                videoinput_set_control_v4l2( vidin, V4L2_CID_AUDIO_VOLUME, ((double) vidin->volume) / 100.0 );
             }
         } else {
             struct video_audio audio;
@@ -1113,7 +1155,9 @@ static void videoinput_do_mute( videoinput_t *vidin, int mute )
                 } else {
                     audio.flags &= ~VIDEO_AUDIO_MUTE;
                 }
-                audio.volume = 60000;
+                if( vidin->volume > 0 ) {
+                    audio.volume = (vidin->volume * 65535) / 100;
+                }
 
                 if( ioctl( vidin->grab_fd, VIDIOCSAUDIO, &audio ) < 0 ) {
                     fprintf( stderr, "videoinput: Can't set audio settings.  I have no idea what "
@@ -1202,6 +1246,7 @@ void videoinput_set_tuner_freq( videoinput_t *vidin, int freqKHz )
         }
 
         vidin->change_muted = 1;
+        mixer_mute( 1 );
         videoinput_do_mute( vidin, vidin->user_muted || vidin->change_muted );
         vidin->cur_tuner_state = TUNER_STATE_SIGNAL_DETECTED;
         vidin->signal_acquire_wait = SIGNAL_ACQUIRE_DELAY;
@@ -1278,7 +1323,7 @@ int videoinput_freq_present( videoinput_t *vidin )
             tuner.index = vidin->tunerid;
             if( ioctl( vidin->grab_fd, VIDIOC_G_TUNER, &tuner ) < 0 ) {
                 if( vidin->verbose ) {
-                    fprintf( stderr, "videoinput: Can't detect signal from tuner: %s\n",
+                    fprintf( stderr, "videoinput: Cannot detect signal from tuner: %s\n",
                              strerror( errno ) );
                 }
             } else if( !tuner.signal ) {
@@ -1289,7 +1334,7 @@ int videoinput_freq_present( videoinput_t *vidin )
 
             if( ioctl( vidin->grab_fd, VIDIOCGTUNER, &tuner ) < 0 ) {
                 if( vidin->verbose ) {
-                    fprintf( stderr, "videoinput: Can't detect signal from tuner: %s\n",
+                    fprintf( stderr, "videoinput: Cannot detect signal from tuner: %s\n",
                              strerror( errno ) );
                 }
             } else if( !tuner.signal ) {
@@ -1472,6 +1517,9 @@ void videoinput_set_input_num( videoinput_t *vidin, int inputnum )
                      strerror( errno ) );
         } else {
             std = videoinput_get_v4l2_norm( vidin->norm );
+            if( std == V4L2_STD_PAL && vidin->dkmode ) {
+                std = V4L2_STD_PAL_DK;
+            } 
             if( ioctl( vidin->grab_fd, VIDIOC_S_STD, &std ) < 0 ) {
                 fprintf( stderr, "videoinput: Driver refuses to set norm: %s\n",
                          strerror( errno ) );
@@ -1542,6 +1590,7 @@ int videoinput_check_for_signal( videoinput_t *vidin, int check_freq_present )
             if( vidin->change_muted ) {
                 vidin->change_muted = 0;
                 videoinput_do_mute( vidin, vidin->user_muted || vidin->change_muted );
+                mixer_mute( 0 );
             }
             break;
         }
@@ -1552,6 +1601,7 @@ int videoinput_check_for_signal( videoinput_t *vidin, int check_freq_present )
             vidin->cur_tuner_state = TUNER_STATE_SIGNAL_LOST;
             vidin->signal_recover_wait = SIGNAL_RECOVER_DELAY;
             vidin->change_muted = 1;
+            mixer_mute( 1 );
             videoinput_do_mute( vidin, vidin->user_muted || vidin->change_muted );
         case TUNER_STATE_SIGNAL_LOST:
             if( vidin->signal_recover_wait ) {
@@ -1605,6 +1655,20 @@ int videoinput_get_audio_mode( videoinput_t *vidin )
     }
 }
 
+void videoinput_set_pal_audio_mode( videoinput_t *vidin, int dkmode )
+{
+    if( dkmode != vidin->dkmode && vidin->norm == VIDEOINPUT_PAL ) {
+        vidin->dkmode = dkmode;
+        videoinput_set_input_num( vidin, videoinput_get_input_num( vidin ) );
+    }
+}
+
+int videoinput_get_pal_audio_mode( videoinput_t *vidin )
+{
+    if( !vidin ) return 0;
+    return vidin->dkmode;
+}
+
 int videoinput_has_tuner( videoinput_t *vidin )
 {
     return vidin->hastuner;
@@ -1620,6 +1684,11 @@ int videoinput_get_height( videoinput_t *vidin )
     return vidin->height;
 }
 
+int videoinput_get_maxwidth( videoinput_t *vidin )
+{
+    return vidin->maxwidth;
+}
+
 int videoinput_get_norm( videoinput_t *vidin )
 {
     return vidin->norm;
@@ -1645,6 +1714,11 @@ int videoinput_is_uyvy( videoinput_t *vidin )
     return vidin->isuyvy;
 }
 
+int videoinput_is_v4l2( videoinput_t *vidin )
+{
+    return vidin->isv4l2;
+}
+
 int videoinput_get_input_num( videoinput_t *vidin )
 {
     return vidin->curinput;
@@ -1660,3 +1734,32 @@ const char *videoinput_get_driver_name( videoinput_t *vidin )
     return vidin->drivername;
 }
 
+void videoinput_set_capture_volume( videoinput_t *vidin, int volume )
+{
+    vidin->volume = volume;
+    if( vidin->volume >= 0 ) {
+        if( vidin->isv4l2 ) {
+            videoinput_set_control_v4l2( vidin, V4L2_CID_AUDIO_VOLUME,
+                                         ((double) vidin->volume) / 100.0 );
+        } else {
+            struct video_audio audio;
+
+            if( ioctl( vidin->grab_fd, VIDIOCGAUDIO, &audio ) < 0 ) {
+                fprintf( stderr, "videoinput: Audio state query failed (got '%s').\n",
+                         strerror( errno ) );
+            } else {
+                if( vidin->volume > 0 ) {
+                    audio.volume = (vidin->volume * 65535) / 100;
+                }
+
+                if( ioctl( vidin->grab_fd, VIDIOCSAUDIO, &audio ) < 0 ) {
+                    fprintf( stderr, "videoinput: Can't set audio settings.  I have no idea what "
+                             "might cause this.  Post a bug report with your driver info to "
+                             PACKAGE_BUGREPORT "\n" );
+                    fprintf( stderr, "videoinput: Include this error: '%s'\n", strerror( errno ) );
+                }
+            }
+        }
+    }
+}
+
