@@ -57,6 +57,10 @@
 /* Every 30 seconds, ping the screensaver. */
 #define SCREENSAVER_PING_TIME (30 * 1000 * 1000)
 
+/* Useful. */
+#define MIN(x,y)((x)<(y)?(x):(y))
+#define MAX(x,y)((x)>(y)?(x):(y))
+
 static Display *display;
 static int screen;
 static Window wm_window;
@@ -91,10 +95,16 @@ static Atom net_wm_state;
 static Atom net_wm_state_above;
 static Atom net_wm_state_below;
 static Atom net_wm_state_fullscreen;
+static Atom net_wm_user_time;
+static Atom net_wm_icon;
+static Atom net_active_window;
 static Atom utf8_string;
+static Atom cardinal;
 static Atom kwm_keep_on_top;
 static Atom wm_protocols;
 static Atom wm_delete_window;
+static Atom xawtv_station;
+static Atom xawtv_remote;
 
 #ifdef HAVE_XTESTEXTENSION
 static KeyCode kc_shift_l; /* Fake key to send. */
@@ -104,6 +114,10 @@ static area_t video_area;
 static area_t window_area;
 static area_t scale_area;
 
+static Time lastpresstime = 0;
+static Time lastreleasetime = 0;
+static Time last_server_time = 0;
+
 static int timediff( struct timeval *large, struct timeval *small )
 {
     return (   ( ( large->tv_sec * 1000 * 1000 ) + large->tv_usec )
@@ -126,14 +140,20 @@ static void load_atoms( Display *dpy )
         "_NET_WM_STATE_ABOVE",
         "_NET_WM_STATE_BELOW",
         "_NET_WM_STATE_FULLSCREEN",
+        "_NET_WM_USER_TIME",
+        "_NET_WM_ICON",
+        "_NET_ACTIVE_WINDOW",
         "UTF8_STRING",
+        "CARDINAL",
         "KWM_KEEP_ON_TOP",
         "WM_PROTOCOLS",
-        "WM_DELETE_WINDOW"
+        "WM_DELETE_WINDOW",
+        "_XAWTV_STATION",
+        "_XAWTV_REMOTE"
     };
-    Atom atoms_return[ 11 ];
+    Atom atoms_return[ 17 ];
 
-    XInternAtoms( display, atom_names, 11, False, atoms_return );
+    XInternAtoms( display, atom_names, 17, False, atoms_return );
     net_supporting_wm_check = atoms_return[ 0 ];
     net_supported = atoms_return[ 1 ];
     net_wm_name = atoms_return[ 2 ];
@@ -141,12 +161,80 @@ static void load_atoms( Display *dpy )
     net_wm_state_above = atoms_return[ 4 ];
     net_wm_state_below = atoms_return[ 5 ];
     net_wm_state_fullscreen = atoms_return[ 6 ];
-    utf8_string = atoms_return[ 7 ];
-    kwm_keep_on_top = atoms_return[ 8 ];
-    wm_protocols = atoms_return[ 9 ];
-    wm_delete_window = atoms_return[ 10 ];
+    net_wm_user_time = atoms_return[ 7 ];
+    net_wm_icon = atoms_return[ 8 ];
+    net_active_window = atoms_return[ 9 ];
+    utf8_string = atoms_return[ 10 ];
+    cardinal = atoms_return[ 11 ];
+    kwm_keep_on_top = atoms_return[ 12 ];
+    wm_protocols = atoms_return[ 13 ];
+    wm_delete_window = atoms_return[ 14 ];
+    xawtv_station = atoms_return[ 15 ];
+    xawtv_remote = atoms_return[ 16 ];
 }
 
+static uint32_t icon_colours[256];
+static int colours_loaded = 0;
+
+static const char *tvtime_icon =
+"                                "
+"                                "
+"                                "
+"                                "
+"    %@@@@@@@++..........%       "
+"    ..####=##=##=####..#*&      "
+"    ++@%@@@@@+++++..+...%&&     "
+"    ..%%%@@@+++....#+.+.%$&$    "
+"    +.@%@@++++...###....%&$&    "
+"    ++@@@@+++..#.#...++.%$&&    "
+"    +.@@+++....#.##...+.%&$&    "
+"    ++@@+++...#.##..+.++%$$&    "
+"    ++++++...###.#..+++.%&$&    "
+"    ++++...#.#.#....++++*$$$    "
+"    ++@....###.#...++++.*$$&    "
+"    +++..#.#.#...+++@+++*$$$    "
+"    @@+.##.#.#...+++@+++%$&$    "
+"    +++.#.##...+++@@@+++*$$$    "
+"    @@.##.#....++@@@%+@+*$$$    "
+"    @@+#.#...++++@@@%+++*&$$    "
+"    @@++....+++@@@%%@+@+-$$&    "
+"    ++++....+++@@%%-%+++*$$     "
+"    @++.+......#...+++++%$$     "
+"      +.++++......#..+@+*$&     "
+"          ...+++++...+++%$      "
+"               ...++@+++*       "
+"                                "
+"                                "
+"                                "
+"                                "
+"                                "
+"                                " ;
+
+static void load_icon( uint32_t *data )
+{
+    int i;
+
+    if( !colours_loaded ) {
+        colours_loaded = 1;
+        icon_colours[ ' ' ] = 0x00000000;
+        icon_colours[ '.' ] = 0xffcccccc;
+        icon_colours[ '+' ] = 0xffb2b2b2;
+        icon_colours[ '@' ] = 0xff999999;
+        icon_colours[ '#' ] = 0xffe5e5e5;
+        icon_colours[ '$' ] = 0xff4c4c4c;
+        icon_colours[ '%' ] = 0xff7f7f7f;
+        icon_colours[ '&' ] = 0xff333333;
+        icon_colours[ '*' ] = 0xff666666;
+        icon_colours[ '=' ] = 0xffffffff;
+        icon_colours[ '-' ] = 0xff007f7f;
+    }
+
+    *data++ = 32;
+    *data++ = 32;
+    for( i = 0; i < 32 * 32; i++ ) {
+        *data++ = icon_colours[ (int) tvtime_icon[ i ] ];
+    }
+}
 
 /**
  * Called after mapping a window - waits until the window is mapped.
@@ -250,33 +338,6 @@ static int xv_get_height_for_width( int window_width )
     return rounded_int_division( window_width * n, d );
 }
 
-static void x11_aspect_hint( Display *dpy, Window win, int aspect_width, int aspect_height )
-{
-    /* These hints don't work with current versions of metacity.
-     * We'll re-enable this code for metacity when newer versions
-     * become more popular.
-     */
-    if( !wm_is_metacity ) {
-        XSizeHints hints;
-
-        hints.flags = PAspect;
-        hints.min_aspect.x = aspect_width;
-        hints.min_aspect.y = aspect_height;
-        hints.max_aspect.x = aspect_width;
-        hints.max_aspect.y = aspect_height;
-
-        XSetWMNormalHints( dpy, win, &hints );
-    }
-}
-
-static void x11_static_gravity( Display *dpy, Window win )
-{
-    XSizeHints hints;
-    hints.flags = PWinGravity;
-    hints.win_gravity = StaticGravity;
-    XSetWMNormalHints( dpy, win, &hints );
-}
-
 static void x11_northwest_gravity( Display *dpy, Window win )
 {
     XSizeHints hints;
@@ -370,7 +431,7 @@ static void get_window_manager_name( Display *dpy, Window wm_window, char **wm_n
                                     &bytes_after_return,
                                     &prop_return ) != Success ) {
                 fprintf( stderr, "wm_name: Can't get window "
-                                 "manager name propety "
+                                 "manager name property "
                                  "(WM not compliant).\n" );
                 *wm_name_return = strdup( "unknown" );
                 return;
@@ -759,7 +820,7 @@ static void calculate_video_area( void )
 }
 
 
-int xcommon_open_display( int aspect, int init_height, int verbose )
+int xcommon_open_display( const char *user_geometry, int aspect, int verbose )
 {
     Pixmap curs_pix;
     XEvent xev;
@@ -770,12 +831,12 @@ int xcommon_open_display( int aspect, int init_height, int verbose )
     const char *hello = "tvtime";
     unsigned long mask;
     char *wmname;
+    int wx, wy;
+    unsigned int ww, wh;
+    int gflags;
 
     output_aspect = aspect;
-    output_height = init_height;
-    if( output_height < 0 ) {
-        output_height = 576;
-    }
+    output_height = 576;
 
     have_xtest = 0;
     output_on_root = 0;
@@ -881,19 +942,35 @@ int xcommon_open_display( int aspect, int init_height, int verbose )
     xswa.backing_store = NotUseful;
     xswa.save_under = False;
     xswa.background_pixel = BlackPixel( display, screen );
-    xswa.event_mask = ButtonPressMask | ButtonReleaseMask | StructureNotifyMask |
-                      KeyPressMask | PointerMotionMask | VisibilityChangeMask |
-                      FocusChangeMask | PropertyChangeMask;
+    xswa.event_mask = ButtonPressMask | ButtonReleaseMask |
+                      StructureNotifyMask | KeyPressMask | PointerMotionMask |
+                      VisibilityChangeMask | FocusChangeMask |
+                      PropertyChangeMask | ExposureMask;
 
     mask = (CWBackPixel | CWSaveUnder | CWBackingStore | CWEventMask);
 
+    gflags = XParseGeometry( user_geometry, &wx, &wy, &ww, &wh );
+    if( !(gflags & WidthValue) ) {
+        ww = output_width;
+    }
+    if( !(gflags & HeightValue) ) {
+        wh = output_height;
+    } else if( ww == 0 ) {
+        ww = xv_get_width_for_height( wh );
+    }
+
+    window_area.width = ww;
+    window_area.height = wh;
+    output_width = ww;
+    output_height = wh;
+
     wm_window = XCreateWindow( display, RootWindow( display, screen ), 0, 0,
-                               output_width, output_height, 0,
-                               CopyFromParent, InputOutput, CopyFromParent, mask, &xswa);
+                               ww, wh, 0, CopyFromParent, InputOutput,
+                               CopyFromParent, mask, &xswa);
 
     output_window = XCreateWindow( display, wm_window, 0, 0,
-                                   output_width, output_height, 0,
-                                   CopyFromParent, InputOutput, CopyFromParent, mask, &xswa);
+                                   ww, wh, 0, CopyFromParent, InputOutput,
+                                   CopyFromParent, mask, &xswa);
 
     xswa.override_redirect = True;
     xswa.border_pixel = 0;
@@ -905,7 +982,7 @@ int xcommon_open_display( int aspect, int init_height, int verbose )
                                CopyFromParent, InputOutput, CopyFromParent,
                                mask, &xswa );
 
-    /* Tell KDE to keep the fullscreen window on top */
+    /* Tell KDE to keep the fullscreen window on top. */
     {
         XEvent ev;
         long mask;
@@ -921,6 +998,21 @@ int xcommon_open_display( int aspect, int init_height, int verbose )
         XSendEvent( display, DefaultRootWindow( display ), False, mask, &ev );
     }
 
+    /* Set the icon on the window. */
+    {
+        uint32_t *data = malloc( 1026 * 4 );
+        if( data ) {
+            load_icon( data );
+            XChangeProperty( display, wm_window, net_wm_icon, cardinal, 32,
+                             PropModeReplace,
+                             (const unsigned char*) data, 1026 );
+            XChangeProperty( display, fs_window, net_wm_icon, cardinal, 32,
+                             PropModeReplace, (const unsigned char*) data,
+                             1026 );
+            free( data );
+        }
+    }
+
     if( check_for_EWMH_wm( display, &wmname ) ) {
         if( xcommon_verbose ) {
             fprintf( stderr, "xcommon: Window manager is %s and is EWMH compliant.\n", wmname );
@@ -971,6 +1063,11 @@ int xcommon_open_display( int aspect, int init_height, int verbose )
     classhint.res_name = "TVFullscreen";
     XSetClassHint( display, fs_window, &classhint );
 
+    /* collaborate with the window manager for close requests */
+    XSetWMProtocols( display, wm_window, &wm_delete_window, 1 );
+
+    calculate_video_area();
+
     XMapWindow( display, output_window );
     XMapWindow( display, wm_window );
 
@@ -992,16 +1089,6 @@ int xcommon_open_display( int aspect, int init_height, int verbose )
     XDefineCursor( display, output_window, nocursor );
     XSetIconName( display, wm_window, "tvtime" );
 
-    /* collaborate with the window manager for close requests */
-    XSetWMProtocols( display, wm_window, &wm_delete_window, 1 );
-
-    calculate_video_area();
-    x11_aspect_hint( display, wm_window, video_area.width, video_area.height );
-
-    if( init_height < 0 ) {
-        xcommon_resize_window_fullscreen();
-    }
-
     return 1;
 }
 
@@ -1056,22 +1143,6 @@ void xcommon_set_window_position( int x, int y )
     XMoveWindow( display, wm_window, x, y );
 }
 
-void xcommon_resize_window_fullscreen( void )
-{
-    int x, y, w, h;
-
-    if( output_fullscreen ) {
-        xcommon_toggle_fullscreen( 0, 0 );
-    }
-
-    xfullscreen_update( xf );
-    xfullscreen_get_position( xf, window_area.x, window_area.y, &x, &y, &w, &h );
-
-    /* Show our fullscreen window. */
-    x11_static_gravity( display, wm_window );
-    XMoveResizeWindow( display, wm_window, x, y, w, h );
-}
-
 void xcommon_set_window_height( int window_height )
 {
     XWindowChanges win_changes;
@@ -1090,8 +1161,8 @@ void xcommon_set_window_height( int window_height )
     win_changes.width = window_width;
     win_changes.height = window_height;
 
-    XResizeWindow( display, wm_window, window_width, window_height );
-    XReconfigureWMWindow( display, wm_window, 0, CWWidth | CWHeight, &win_changes );
+    XReconfigureWMWindow( display, wm_window, 0,
+                          CWWidth | CWHeight, &win_changes );
 }
 
 void xcommon_clear_screen( void )
@@ -1111,6 +1182,28 @@ void xcommon_clear_screen( void )
     XSync( display, False );
 }
 
+void xcommon_clear_area( int x, int y, int w, int h )
+{
+    int dx, dy, dw, dh;
+
+    /* First fill the new area to black. */
+    XSetForeground( display, gc, BlackPixel( display, screen ) );
+    XFillRectangle( display, output_window, gc, x, y, w, h );
+
+    /**
+     * Only fill the intersection of the video area and the exposed
+     * area with the colour key.
+     */
+    dx = MAX( x, video_area.x );
+    dy = MAX( y, video_area.y );
+    dw = MIN( x + w, video_area.x + video_area.width ) - dx;
+    dh = MIN( y + h, video_area.y + video_area.height ) - dy;
+    if( dx >= 0 && dy >= 0 && dw >= 0 && dh >= 0 ) {
+        XSetForeground( display, gc, xcommon_colourkey );
+        XFillRectangle( display, output_window, gc, dx, dy, dw, dh );
+    }
+}
+
 /**
  * Called after unmapping a window - waits until the window is unmapped.
  */
@@ -1144,14 +1237,31 @@ int xcommon_toggle_alwaysontop( void )
     return alwaysontop;
 }
 
+static void xcommon_activate_window( void )
+{
+    XEvent ev;
+
+    ev.type = ClientMessage;
+    ev.xclient.window = wm_window;
+    ev.xclient.message_type = net_active_window;
+    ev.xclient.format = 32;
+    ev.xclient.data.l[ 0 ] = 1;
+    ev.xclient.data.l[ 1 ] = last_server_time;
+    ev.xclient.data.l[ 2 ] = 0;
+
+    XSendEvent( display, DefaultRootWindow( display ), False,
+                SubstructureNotifyMask|SubstructureRedirectMask, &ev );
+}
+
 int xcommon_toggle_fullscreen( int fullscreen_width, int fullscreen_height )
 {
     output_fullscreen = !output_fullscreen;
     if( output_fullscreen ) {
         int x, y, w, h;
-  
+
         xfullscreen_update( xf );
         xfullscreen_get_position( xf, window_area.x, window_area.y,
+                                  window_area.width, window_area.height,
                                   &x, &y, &w, &h );
 
         output_width = w;
@@ -1179,6 +1289,9 @@ int xcommon_toggle_fullscreen( int fullscreen_width, int fullscreen_height )
             XSendEvent( display, DefaultRootWindow( display ), False,
                         SubstructureNotifyMask | SubstructureRedirectMask,
                         &ev );
+
+            /* When we go fullscreen, explicitly ask to get focus. */
+            xcommon_activate_window();
         } else {
             /* Show our fullscreen window. */
             XMoveResizeWindow( display, fs_window, x, y, w, h );
@@ -1248,11 +1361,35 @@ int xcommon_toggle_aspect( void )
 {
     output_aspect = !output_aspect;
     calculate_video_area();
-    x11_aspect_hint( display, wm_window, video_area.width, video_area.height );
     xcommon_clear_screen();
     return output_aspect;
 }
 
+void xcommon_update_server_time( unsigned long timestamp )
+{
+    if( !timestamp ) {
+        /**
+         * Get the current timestamp by causing an event to be generated
+         * by the X server.  This is only a single round-trip to the
+         * X server.  Inspired by kapplication::updateUserTimestamp().
+         */
+        unsigned char data[ 1 ];
+        Window w = XCreateSimpleWindow( display, DefaultRootWindow( display ),
+                                        0, 0, 1, 1, 0, 0, 0 );
+        XEvent ev;
+        XSelectInput( display, w, PropertyChangeMask );
+        XChangeProperty( display, w, XA_ATOM, XA_ATOM, 8,
+                         PropModeAppend, data, 1 );
+        XWindowEvent( display, w, PropertyChangeMask, &ev );
+        timestamp = ev.xproperty.time;
+        XDestroyWindow( display, w );
+    }
+
+    last_server_time = timestamp;
+    XChangeProperty( display, wm_window, net_wm_user_time, XA_CARDINAL, 32,
+                     PropModeReplace, (unsigned char *) &timestamp, 1 );
+}
+
 void xcommon_poll_events( input_t *in )
 {
     XEvent event;
@@ -1263,6 +1400,10 @@ void xcommon_poll_events( input_t *in )
     int motion = 0;
     int motion_x = 0;
     int motion_y = 0;
+    int bx = -1;
+    int by = -1;
+    int bw = -1;
+    int bh = -1;
 
     while( XPending( display ) ) {
         KeySym mykey;
@@ -1280,6 +1421,29 @@ void xcommon_poll_events( input_t *in )
             }
             break;
 
+        case Expose:
+            if( bx < 0 ) {
+                bx = event.xexpose.x;
+                by = event.xexpose.y;
+                bw = event.xexpose.width;
+                bh = event.xexpose.height;
+            } else {
+                int dx, dy, dw, dh;
+                int cx, cy, cw, ch;
+                cx = event.xexpose.x;
+                cy = event.xexpose.y;
+                cw = event.xexpose.width;
+                ch = event.xexpose.height;
+                dx = MIN(bx, cx);
+                dy = MIN(by, cy);
+                dw = MAX(bx + bw, cx + cw) - dx;
+                dh = MAX(by + bh, cy + ch) - dy;
+                bx = dx;
+                by = dy;
+                bw = dw;
+                bh = dh;
+            }
+            break;
         case DestroyNotify:
             if( event.xdestroywindow.window != wm_window ) {
                 break;
@@ -1353,8 +1517,20 @@ void xcommon_poll_events( input_t *in )
             break;
         case ConfigureNotify:
             if( event.xconfigure.window == wm_window ) {
-                window_area.x = event.xconfigure.x;
-                window_area.y = event.xconfigure.y;
+                if( event.xconfigure.send_event == False ) {
+                    int gx, gy;
+                    Window dw;
+
+                    if( XTranslateCoordinates( display, wm_window,
+                                               DefaultRootWindow( display ),
+                                               0, 0, &gx, &gy, &dw) == True) {
+                        window_area.x = gx;
+                        window_area.y = gy;
+                    }
+                } else {
+                    window_area.x = event.xconfigure.x;
+                    window_area.y = event.xconfigure.y;
+                }
                 window_area.width = event.xconfigure.width;
                 window_area.height = event.xconfigure.height;
             }
@@ -1364,7 +1540,35 @@ void xcommon_poll_events( input_t *in )
                 reconfigure = 1;
             }
             break;
+        case PropertyNotify:
+            if( event.xproperty.atom == xawtv_remote ) {
+                Atom type_return;
+                int format_return;
+                unsigned long bytes_after_return;
+                unsigned long nitems_return;
+                unsigned char *prop_return = 0;
+                char *argv[ 32 ];
+                if( XGetWindowProperty( display, event.xproperty.window,
+                                        event.xproperty.atom, 0, 65536,
+                                        True, XA_STRING, &type_return,
+                                        &format_return, &nitems_return,
+                                        &bytes_after_return,
+                                        &prop_return ) == Success ) {
+                    if( nitems_return > 0 ) {
+                        int arg = 0;
+                        int i;
+                        for( i = 0; i < nitems_return && *(prop_return + i); i += strlen( ((char *) prop_return) + i ) + 1 ) {
+                            argv[ arg++ ] = ((char *) prop_return) + i;
+                        }
+                        argv[ arg ] = 0;
+                        input_xawtv_command( in, arg, argv );
+                        XFree( prop_return );
+                    }
+                }
+            }
+            break;
         case KeyPress:
+            xcommon_update_server_time( event.xkey.time );
             /* if( event.xkey.state & ShiftMask ) arg |= I_SHIFT; */
             /* this alternate approach allows handling of keys like '<' and '>' -- mrallen */
             if( event.xkey.state & ShiftMask ) {
@@ -1444,11 +1648,19 @@ void xcommon_poll_events( input_t *in )
             input_callback( in, I_KEYDOWN, arg );
             break;
         case ButtonPress:
-            input_callback( in, I_BUTTONPRESS, event.xbutton.button );
+            xcommon_update_server_time( event.xbutton.time );
+            if( event.xbutton.time != lastpresstime ) {
+                input_callback( in, I_BUTTONPRESS, event.xbutton.button );
+                lastpresstime = event.xbutton.time;
+            }
             getfocus = 1;
             break;
         case ButtonRelease:
-            input_callback( in, I_BUTTONRELEASE, event.xbutton.button );
+            xcommon_update_server_time( event.xbutton.time );
+            if( event.xbutton.time != lastreleasetime ) {
+                input_callback( in, I_BUTTONRELEASE, event.xbutton.button );
+                lastreleasetime = event.xbutton.time;
+            }
             getfocus = 1;
             break;
         default: break;
@@ -1478,6 +1690,10 @@ void xcommon_poll_events( input_t *in )
         }
     }
 
+    if( bx >= 0 ) {
+        xcommon_clear_area( bx, by, bw, bh );
+    }
+
     if( reconfigure ) {
         if( !output_on_root && !output_fullscreen ) {
             output_width = reconfwidth;
@@ -1595,3 +1811,18 @@ void xcommon_set_video_scale( area_t scalearea )
     scale_area = scalearea;
 }
 
+void xcommon_update_xawtv_station( int frequency, int channel_id,
+                                   const char *channel_name )
+{
+    char data[ 1024 ];
+    int length;
+
+    length  = sprintf( data, "%.3f", ((float) frequency) / 1000.0 ) + 1;
+    length += sprintf( data + length, "%d", channel_id ) + 1;
+    length += sprintf( data + length, "%s", channel_name ) + 1;
+
+    XChangeProperty( display, wm_window, xawtv_station,
+                     XA_STRING, 8, PropModeReplace,
+                     (unsigned char *) data, length );
+}
+
